from django.test import TestCase
from model_bakery import baker

from bpp.models import Autor
from bpp.util import get_copy_from_db, has_changed, slugify_function


class TestUtil(TestCase):
    def test_slugify_function(self):
        test = "Waldemar A. Łącki,,()*':;\r\n[]"
        result = "Waldemar-A-Lacki"
        self.assertEqual(slugify_function(test), result)

    def test_slugify_function_double_dash(self):
        test = "Andrzej   Wróbel"
        result = "Andrzej-Wrobel"
        self.assertEqual(slugify_function(test), result)

    def test_get_copy_from_db(self):
        a = baker.make(Autor)
        b = get_copy_from_db(a)
        self.assertEqual(a.pk, b.pk)

    def test_has_changed(self):
        a = baker.make(Autor)
        self.assertEqual(has_changed(a, "nazwisko"), False)

        a.nazwisko = "Foo"
        self.assertEqual(has_changed(a, "nazwisko"), True)

        a.save()
        self.assertEqual(has_changed(a, ["nazwisko", "imiona"]), False)

        a.imiona = "Bar"
        self.assertEqual(has_changed(a, ["nazwisko", "imiona"]), True)



# Generated by Django 3.0.11 on 2021-03-08 11:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0244_auto_20210307_2329"),
    ]

    operations = [
        migrations.AlterField(
            model_name="typ_kbn",
            name="charakter_pbn",
            field=models.ForeignKey(
                blank=True,
                default=None,
                help_text="Wartość wybrana w tym polu zostanie użyta jako\n        fallback, tzn. jeżeli dla charakteru formalnego danego rekordu nie\n        określono odpowiedniego charakteru PBN, to zostanie użyta wartość\n        tego pola, o ile wybrana. ",
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="bpp.Charakter_PBN",
                verbose_name="Charakter PBN",
            ),
        ),
        migrations.AlterField(
            model_name="typ_odpowiedzialnosci",
            name="typ_ogolny",
            field=models.SmallIntegerField(
                choices=[
                    (0, "autor"),
                    (1, "redaktor"),
                    (2, "inny"),
                    (3, "tłumacz"),
                    (4, "komentator"),
                    (5, "recenzent"),
                    (6, "opracował"),
                ],
                db_index=True,
                default=0,
                help_text="Pole to jest używane celem rozróżniania typu odpowiedzialności\n        na cele eksportu do PBN (autor i redaktor) oraz może być też wykorzystywane\n        np. w raportach autorów i jednostek.\n        ",
                verbose_name="Ogólny typ odpowiedzialności",
            ),
        ),
    ]



from bpp.admin.autor_dyscyplina import Autor_DyscyplinaResource
from bpp.models import Autor_Dyscyplina


def test_autor_dyscyplina_resource(autor_jan_kowalski, dyscyplina1):
    Autor_Dyscyplina.objects.create(
        autor=autor_jan_kowalski, rok=2020, dyscyplina_naukowa=dyscyplina1
    )
    assert "Kowalski" in str(Autor_DyscyplinaResource().export())



# -*- encoding: utf-8 -*-

from django import template


register = template.Library()


@register.filter
def just_single_quotes(value):
    if value:
        return value.replace("'", "&#39;")
    return value


# Generated by Django 3.0.7 on 2020-06-28 21:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0213_merge_20200607_2047"),
    ]

    operations = [
        migrations.AddField(
            model_name="patent",
            name="nie_eksportuj_przez_api",
            field=models.BooleanField(
                db_index=True,
                default=False,
                help_text="Jeżeli zaznaczone, to ten rekord nie będzie dostępny przez JSON REST API",
                verbose_name="Nie eksportuj przez API",
            ),
        ),
        migrations.AddField(
            model_name="praca_doktorska",
            name="nie_eksportuj_przez_api",
            field=models.BooleanField(
                db_index=True,
                default=False,
                help_text="Jeżeli zaznaczone, to ten rekord nie będzie dostępny przez JSON REST API",
                verbose_name="Nie eksportuj przez API",
            ),
        ),
        migrations.AddField(
            model_name="praca_habilitacyjna",
            name="nie_eksportuj_przez_api",
            field=models.BooleanField(
                db_index=True,
                default=False,
                help_text="Jeżeli zaznaczone, to ten rekord nie będzie dostępny przez JSON REST API",
                verbose_name="Nie eksportuj przez API",
            ),
        ),
        migrations.AddField(
            model_name="wydawnictwo_ciagle",
            name="nie_eksportuj_przez_api",
            field=models.BooleanField(
                db_index=True,
                default=False,
                help_text="Jeżeli zaznaczone, to ten rekord nie będzie dostępny przez JSON REST API",
                verbose_name="Nie eksportuj przez API",
            ),
        ),
        migrations.AddField(
            model_name="wydawnictwo_zwarte",
            name="nie_eksportuj_przez_api",
            field=models.BooleanField(
                db_index=True,
                default=False,
                help_text="Jeżeli zaznaczone, to ten rekord nie będzie dostępny przez JSON REST API",
                verbose_name="Nie eksportuj przez API",
            ),
        ),
    ]



# Generated by Django 3.0.11 on 2021-04-06 02:01

from django.db import migrations, models

import django.contrib.postgres.fields.jsonb


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Country",
            fields=[
                ("created_on", models.DateTimeField(auto_now_add=True)),
                ("last_updated_on", models.DateTimeField(auto_now=True)),
                (
                    "code",
                    models.CharField(max_length=5, primary_key=True, serialize=False),
                ),
                ("description", models.CharField(max_length=200)),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="Language",
            fields=[
                ("created_on", models.DateTimeField(auto_now_add=True)),
                ("last_updated_on", models.DateTimeField(auto_now=True)),
                (
                    "code",
                    models.CharField(max_length=5, primary_key=True, serialize=False),
                ),
                ("language", django.contrib.postgres.fields.jsonb.JSONField()),
            ],
            options={
                "abstract": False,
            },
        ),
    ]



from braces.views import GroupRequiredMixin, JSONResponseMixin
from celery import uuid
from celery.result import AsyncResult
from django.db import transaction
from django.db.models import Q
from django.http import HttpResponseRedirect
from django.urls import reverse, reverse_lazy
from django.views.generic import CreateView, DetailView, ListView, UpdateView
from django.views.generic.detail import BaseDetailView
from django.views.generic.edit import BaseDeleteView

from import_dyscyplin.forms import (
    Import_Dyscyplin_KolumnaForm,
    KolumnaFormSet,
    KolumnaFormSetHelper,
)
from import_dyscyplin.tasks import (
    integruj_import_dyscyplin,
    przeanalizuj_import_dyscyplin,
    stworz_kolumny,
)
from notifications.mixins import ChannelSubscriberSingleObjectMixin
from .forms import Import_DyscyplinForm
from .models import Import_Dyscyplin

from django.contrib import messages

from bpp.const import GR_WPROWADZANIE_DANYCH


class WprowadzanieDanychRequiredMixin(GroupRequiredMixin):
    group_required = GR_WPROWADZANIE_DANYCH


class TylkoMojeMixin:
    def get_queryset(self):
        return Import_Dyscyplin.objects.filter(owner=self.request.user)


class ListImport_Dyscyplin(GroupRequiredMixin, TylkoMojeMixin, ListView):
    group_required = GR_WPROWADZANIE_DANYCH
    model = Import_Dyscyplin


class CreateImport_Dyscyplin(GroupRequiredMixin, CreateView):
    group_required = GR_WPROWADZANIE_DANYCH
    model = Import_Dyscyplin
    form_class = Import_DyscyplinForm

    def get_success_url(self):
        return reverse("import_dyscyplin:detail", args=(self.object.pk,))

    def form_valid(self, form):
        self.object = form.save(commit=False)
        self.object.owner = self.request.user
        self.object.save()
        return HttpResponseRedirect(self.get_success_url())


class KolumnyImport_Dyscyplin(
    GroupRequiredMixin, TylkoMojeMixin, ChannelSubscriberSingleObjectMixin, UpdateView
):
    group_required = GR_WPROWADZANIE_DANYCH
    model = Import_Dyscyplin
    template_name = "import_dyscyplin/import_dyscyplin_kolumny.html"
    form_class = Import_Dyscyplin_KolumnaForm

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        if self.request.POST:
            context["kolumny_formset"] = KolumnaFormSet(
                self.request.POST, instance=self.object
            )
            context["kolumny_formset"].full_clean()
        else:
            context["kolumny_formset"] = KolumnaFormSet(instance=self.object)
        context["kolumny_formset_helper"] = KolumnaFormSetHelper()
        return context

    @transaction.atomic
    def form_valid(self, form):
        context = self.get_context_data(form=form)
        formset = context["kolumny_formset"]
        if formset.is_valid():
            self.object.zatwierdz_kolumny()

            response = super().form_valid(form)

            formset.instance = self.object
            formset.save()
            return response
        else:
            return super().form_invalid(form)

    def get_success_url(self):
        return reverse("import_dyscyplin:detail", args=(self.object.pk,))


class DetailImport_Dyscyplin(
    GroupRequiredMixin, TylkoMojeMixin, ChannelSubscriberSingleObjectMixin, DetailView
):
    group_required = GR_WPROWADZANIE_DANYCH
    model = Import_Dyscyplin

    def get_context_data(self, **kwargs):
        return super().get_context_data(
            notification=self.request.GET.get("notification", "0"), **kwargs
        )


class UruchomZadaniePrzetwarzania(
    GroupRequiredMixin, TylkoMojeMixin, DetailView, JSONResponseMixin
):
    group_required = GR_WPROWADZANIE_DANYCH
    model = Import_Dyscyplin
    task = None
    stan = None

    def get_queryset(self):
        return super(TylkoMojeMixin, self).get_queryset().filter(stan=self.stan)

    def get(self, *args, **kw):
        self.object = self.get_object()

        start_task = False
        if (
            self.object.task_id is None
            or AsyncResult(self.object.task_id).status == "PENDING"
        ):
            start_task = True
            self.object.task_id = task_id = uuid()

        self.object.save()

        if start_task:
            transaction.on_commit(
                lambda self=self: self.task.apply_async(
                    args=(self.object.pk,), task_id=task_id
                )
            )

        return self.render_json_response({"status": "ok"})


class UruchomTworzenieKolumnImport_Dyscyplin(UruchomZadaniePrzetwarzania):
    group_required = GR_WPROWADZANIE_DANYCH
    model = Import_Dyscyplin
    task = stworz_kolumny
    stan = Import_Dyscyplin.STAN.NOWY


class UruchomPrzetwarzanieImport_Dyscyplin(UruchomZadaniePrzetwarzania):
    group_required = GR_WPROWADZANIE_DANYCH
    model = Import_Dyscyplin
    task = przeanalizuj_import_dyscyplin
    stan = Import_Dyscyplin.STAN.OPCJE_IMPORTU_OKRESLONE


class UruchomIntegracjeImport_DyscyplinView(UruchomZadaniePrzetwarzania):
    group_required = GR_WPROWADZANIE_DANYCH
    model = Import_Dyscyplin
    task = integruj_import_dyscyplin
    stan = Import_Dyscyplin.STAN.PRZEANALIZOWANY


class UsunImport_Dyscyplin(
    WprowadzanieDanychRequiredMixin, TylkoMojeMixin, BaseDeleteView
):
    success_url = reverse_lazy("import_dyscyplin:index")

    def delete(self, request, *args, **kwargs):
        self.object = self.get_object()
        transaction.on_commit(
            lambda: messages.add_message(
                self.request,
                messages.INFO,
                'Plik importu dyscyplin "%s" został usunięty.' % self.object.plik.name,
            )
        )
        return super().delete(request, *args, **kwargs)

    get = delete


class API_Do_IntegracjiView(
    WprowadzanieDanychRequiredMixin, TylkoMojeMixin, JSONResponseMixin, BaseDetailView
):
    func = "poprawne_wiersze_do_integracji"

    def get(self, *args, **kw):
        self.object = self.get_object()

        fn = getattr(self.object, self.func)()

        fn_filtered = fn
        search = self.request.GET.get("search[value]", "")
        if search:
            fn_filtered = fn_filtered.filter(
                Q(nazwisko__icontains=search)
                | Q(imiona__icontains=search)
                | Q(original__nazwa_jednostki__icontains=search)
                | Q(original__wydział__icontains=search)
                | Q(info__icontains=search)
                | Q(dyscyplina__icontains=search)
                | Q(subdyscyplina__icontains=search)
                | Q(kod_dyscypliny__icontains=search)
                | Q(kod_subdyscypliny__icontains=search)
            )

        start = int(self.request.GET.get("start", 0))
        draw = int(self.request.GET.get("draw", 1))
        length = int(self.request.GET.get("length", 10))

        ordering = int(self.request.GET.get("order[0][column]", 0))
        direction = self.request.GET.get("order[0][dir]", "asc")
        fld = self.request.GET.get("columns[%i][data]" % ordering, "dopasowanie_autora")
        if fld:
            ordering_mapping = {
                "dopasowanie_autora": "autor__nazwisko",
                "jednostka": "nazwa_jednostki",
                "wydzial": "nazwa_wydzialu",
                "dyscyplina": "dyscyplina__nazwa",
                "procent_dyscypliny": "procent_dyscypliny",
                "subdyscyplina": "subdyscyplina__nazwa",
                "procent_subdyscypliny": "procent_subdyscypliny",
            }
            fld = ordering_mapping.get(fld, fld)
        if direction != "asc":
            fld = "-" + fld

        fn_output = fn_filtered
        if fld:
            fn_output = fn_output.order_by(fld)
        fn_output = fn_output[start : start + length]

        recordsTotal = fn.count()
        recordsFiltered = fn_filtered.count()
        return self.render_json_response(
            {
                "data": [x.serialize_dict() for x in fn_output],
                "draw": draw,
                "recordsTotal": recordsTotal,
                "recordsFiltered": recordsFiltered,
            }
        )


class API_Nie_Do_IntegracjiView(API_Do_IntegracjiView):
    func = "niepoprawne_wiersze"


class API_Zintegrowane(API_Do_IntegracjiView):
    func = "zintegrowane_wiersze"



from __future__ import annotations

import logging
import operator
from functools import reduce

import openpyxl
from django.core.management import BaseCommand
from django.db import transaction
from django.db.models import Q
from flexible_reports.models import Datasource
from openpyxl.styles import Alignment

from import_common.normalization import normalize_filename

from bpp.models import Jednostka, Rekord, Wydzial
from bpp.util import pbar, worksheet_columns_autosize, worksheet_create_table

logger = logging.getLogger("django")


class Command(BaseCommand):
    help = (
        "Polecenie generuje skoroszyty XLSX, po jednym dla każdej jednostki, nazwane od nazw jednostek, "
        "przydzielając prace do plików wg pierwszego afiliowanego autora, z pracami wg zapytań odpowiadających "
        "zapytaniom z raportu wydziałów (flexible_reports/datasources) dla działu 1.x oraz 2.x tych zapytań, dodatkowo "
        "doklejając do skoroszytów kolumny dotyczące opłat za publikacje, do wypełnienia przez kierowników "
        "jednostek. "
    )

    @transaction.atomic
    def handle(self, **options):
        datasources: [Datasource] = (
            Datasource.objects.filter(
                Q(label__startswith="1.1")
                | Q(label__startswith="1.2")
                | Q(label__startswith="2.1")
                | Q(label__startswith="2.3")
            )
            .exclude(label__endswith="jednostka")
            .exclude(label__endswith="autor")
        )

        queries = []
        for wydzial in Wydzial.objects.all():

            class FakeObj:
                pk = wydzial.pk

            for datasource in datasources:
                queries.append(datasource.get_filter({"obiekt": FakeObj()}))

        rekordy = (
            Rekord.objects.filter(reduce(operator.or_, queries))
            .filter(rok__gte=2020)
            .distinct()
        )

        pierwszy_autor_afiliowany = {
            rekord.pk: rekord.pierwszy_autor_afiliowany
            for rekord in rekordy
            if rekord.pierwszy_autor_afiliowany is not None
        }

        jednostki_ids = Jednostka.objects.filter(
            pk__in={paa.jednostka_id for paa in pierwszy_autor_afiliowany.values()}
        ).values_list("pk", flat=True)

        for jednostka_id in pbar(jednostki_ids):
            rekordy_jednostki = rekordy.filter(
                autorzy__pk__in=[
                    paa.pk
                    for paa in pierwszy_autor_afiliowany.values()
                    if paa.jednostka_id == jednostka_id
                ]
            ).order_by("-rok")

            wb = openpyxl.Workbook()
            s = wb.worksheets[0]
            s.append(
                [
                    "Tytuł oryginalny",
                    "ID rekordu",
                    "Rok",
                    "Pierwszy autor afiliowany",
                    "Publikacja bezkosztowa (brak kosztów zewnętrznych\nlub wydawnictwo własne): tak/nie",
                    "Środki finansowe, o których mowa w artykule 365 pkt 2\nustawy (subwencja, w tym rezerwa "
                    "rektora, dziekana,\nopen acess): tak/nie",
                    "Środki finansowe na realizację projektu: tak/nie\n (środki finansowe przyznane na realizację "
                    "projektu\nw zakresie badan naukowych\n lub prac rozwojowych)",
                    "Inne środki finansowe (np. prace \nzlecone): tak/nie",
                    "Kwota (zł brutto)",
                ]
            )

            for rekord in rekordy_jednostki:
                s.append(
                    [
                        rekord.tytul_oryginalny,
                        str(rekord.pk),
                        rekord.rok,
                        str(pierwszy_autor_afiliowany[rekord.id].autor),
                    ]
                )

            worksheet_create_table(s)
            worksheet_columns_autosize(s, max_width=80, multiplier=1.0)

            for row in s:
                for cell in row:
                    cell.alignment = Alignment(wrapText=True)
                    cell.value = cell.value
                break

            c = s["B2"]
            s.freeze_panes = c

            wb.save(
                normalize_filename(Jednostka.objects.get(pk=jednostka_id).nazwa)
                + ".xlsx"
            )



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-04-15 20:37
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0125_auto_20180415_2223'),
        ('import_dyscyplin', '0011_auto_20180415_2223'),
    ]

    operations = [
        migrations.AddField(
            model_name='import_dyscyplin_row',
            name='dyscyplina_ostateczna',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='bpp.Dyscyplina_Naukowa'),
        ),
    ]



from datetime import timedelta

from django.test import TestCase
from django.urls import reverse
from django.utils.timezone import localtime


def test_rest_api_zrodlo_detail(client, zrodlo):
    res = client.get(reverse("api_v1:zrodlo-detail", args=(zrodlo.pk,)))
    assert res.status_code == 200


def test_rest_api_zrodlo_list(client, zrodlo):
    res = client.get(reverse("api_v1:zrodlo-list"))
    assert res.status_code == 200


def test_rest_api_zrodlo_filtering_1(api_client, zrodlo):
    czas = localtime(zrodlo.ostatnio_zmieniony).strftime("%Y-%m-%d %H:%M:%S")

    res = api_client.get(
        reverse("api_v1:zrodlo-list") + f"?ostatnio_zmieniony_after={czas}"
    )
    assert res.json()["count"] == 1


def test_rest_api_zrodlo_filtering_2(api_client, zrodlo):

    czas = localtime(zrodlo.ostatnio_zmieniony + timedelta(seconds=1)).strftime(
        "%Y-%m-%d %H:%M:%S"
    )

    res = api_client.get(
        reverse("api_v1:zrodlo-list") + f"?ostatnio_zmieniony_after={czas}"
    )
    assert res.json()["count"] == 0



import pytest
from dbtemplates.models import Template
from django.urls import reverse

from django.contrib.contenttypes.models import ContentType

from bpp.models import Wydawnictwo_Ciagle
from bpp.models.szablondlaopisubibliograficznego import SzablonDlaOpisuBibliograficznego


@pytest.mark.django_db
def test_BppTemplateAdmin_templatka_zmienia_rekordy(
    admin_app, wydawnictwo_zwarte, denorms, wydawnictwo_ciagle
):
    WERSJA_1 = "werjsa 1"
    WERSJA_2 = "wersja 2"

    t, _ign = Template.objects.update_or_create(
        name="opis_bibliograficzny.html",
        defaults={"content": WERSJA_1},
    )
    SzablonDlaOpisuBibliograficznego.objects.create(
        model=ContentType.objects.get_for_model(Wydawnictwo_Ciagle),
        template=t,
    )
    denorms.rebuild_instances_of(Wydawnictwo_Ciagle)
    denorms.flush()

    wydawnictwo_ciagle.refresh_from_db()
    assert wydawnictwo_ciagle.opis_bibliograficzny_cache == WERSJA_1
    #
    # Koniec inicjalizacji -- właściwy test poniżej
    #

    url = reverse("admin:dbtemplates_template_change", args=(t.pk,))
    res = admin_app.get(url)
    res.forms["template_form"]["content"] = WERSJA_2
    res.forms["template_form"]["name"] = "nazwa.html"
    res = res.forms["template_form"].submit().maybe_follow()

    denorms.flush()

    wydawnictwo_ciagle.refresh_from_db()
    assert wydawnictwo_ciagle.opis_bibliograficzny_cache == WERSJA_2


@pytest.mark.django_db
def test_BppTemplateAdmin_zmiana_szablonu_zmienia_rekordy(
    admin_app, denorms, wydawnictwo_ciagle
):
    WERSJA_1 = "werjsa 1"
    WERSJA_2 = "wersja 2"

    t1, _ign = Template.objects.update_or_create(
        name="opis_bibliograficzny.html",
        defaults={"content": WERSJA_1},
    )
    t2, _ign = Template.objects.update_or_create(
        name="opis_bibliograficzny_2.html", defaults={"content": WERSJA_2}
    )

    szablon = SzablonDlaOpisuBibliograficznego.objects.create(
        model=ContentType.objects.get_for_model(Wydawnictwo_Ciagle),
        template=t1,
    )
    denorms.rebuildall()

    wydawnictwo_ciagle.refresh_from_db()
    assert wydawnictwo_ciagle.opis_bibliograficzny_cache == WERSJA_1

    #
    # Koniec inicjalizacji -- właściwy test poniżej
    #

    url = reverse(
        "admin:bpp_szablondlaopisubibliograficznego_change", args=(szablon.pk,)
    )
    res = admin_app.get(url)

    form = res.forms["szablondlaopisubibliograficznego_form"]
    form["template"].value = t2.pk
    res = form.submit().maybe_follow()

    denorms.flush()

    wydawnictwo_ciagle.refresh_from_db()
    assert wydawnictwo_ciagle.opis_bibliograficzny_cache == WERSJA_2


@pytest.fixture
def typowy_szablon_opisu():
    t, _ign = Template.objects.update_or_create(
        name="opis_bibliograficzny.html",
        defaults={"content": "123"},
    )
    SzablonDlaOpisuBibliograficznego.objects.update_or_create(
        model=None,
        template=t,
    )
    return t


@pytest.mark.django_db
def test_dbtemplates_TemplateAdmin_preview_good(
    typowy_szablon_opisu, admin_client, wydawnictwo_ciagle
):

    TEST_STR = b"456"
    res = admin_client.get(
        "/admin/dbtemplates/template/preview/", {"template": TEST_STR}, follow=True
    )
    assert TEST_STR in res.content


@pytest.mark.django_db
def test_dbtemplates_TemplateAdmin_preview_bad(typowy_szablon_opisu, admin_client):

    res = admin_client.get(
        "/admin/dbtemplates/template/preview/", {"template": "{% if foobar %}"}
    )
    assert "Wystąpił błąd" in res.rendered_content



# Generated by Django 3.0.11 on 2021-01-01 16:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0231_ukryj_status_korekty"),
    ]

    operations = [
        migrations.AlterField(
            model_name="autor",
            name="pseudonim",
            field=models.CharField(
                blank=True,
                help_text="\n    Jeżeli w bazie danych znajdują się autorzy o zbliżonych imionach, nazwiskach i tytułach naukowych,\n    skorzystaj z tego pola aby ułatwić ich rozróżnienie. Pseudonim pokaże się w polach wyszukiwania\n    oraz na podstronie autora, po nazwisku i tytule naukowym.",
                max_length=300,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="uczelnia",
            name="sortuj_jednostki_alfabetycznie",
            field=models.BooleanField(
                default=True,
                help_text="Jeżeli ustawione na 'FAŁSZ', sortowanie jednostek będzie odbywało się ręcznie\n        tzn za pomocą ustalonej przez administratora systemu kolejności. ",
            ),
        ),
    ]



import uuid
from functools import update_wrapper

from django.conf import settings
from django.db import transaction
from django.http import HttpResponseRedirect
from django.shortcuts import render
from sentry_sdk import capture_exception
from templated_email import send_templated_mail

from zglos_publikacje.models import Zgloszenie_Publikacji, Zgloszenie_Publikacji_Autor
from .filters import WydzialJednostkiPierwszegoAutora
from .forms import ZwrocEmailForm

from django.contrib import admin, messages
from django.contrib.admin import helpers
from django.contrib.admin.templatetags.admin_urls import add_preserved_filters

from bpp.admin.helpers import MODEL_Z_OPLATA_ZA_PUBLIKACJE


class Zgloszenie_Publikacji_AutorInline(admin.StackedInline):
    model = Zgloszenie_Publikacji_Autor
    # form = Zgloszenie_Publikacji_AutorInlineForm
    # readonly_fields = ["autor", "jednostka", "dyscyplina_naukowa"]
    fields = ["autor", "jednostka", "dyscyplina_naukowa"]
    extra = 0


@admin.register(Zgloszenie_Publikacji)
class Zgloszenie_PublikacjiAdmin(admin.ModelAdmin):
    list_display = [
        "tytul_oryginalny",
        "utworzono",
        "ostatnio_zmieniony",
        "wydzial_pierwszego_autora",
        "email",
        "status",
    ]
    list_filter = [
        "status",
        "email",
        WydzialJednostkiPierwszegoAutora,
        "rodzaj_zglaszanej_publikacji",
        "rok",
    ]

    fields = (
        (
            "tytul_oryginalny",
            "rok",
            "rodzaj_zglaszanej_publikacji",
        )
        + MODEL_Z_OPLATA_ZA_PUBLIKACJE
        + (
            "email",
            "strona_www",
            "plik",
            "status",
            "przyczyna_zwrotu",
            "kod_do_edycji",
        )
    )

    inlines = [
        Zgloszenie_Publikacji_AutorInline,
    ]

    def has_add_permission(self, request):
        return False

    def has_change_permission(self, request, obj=None):
        return False

    def get_urls(self):
        from django.conf.urls import url

        def wrap(view):
            def wrapper(*args, **kwargs):
                return self.admin_site.admin_view(view)(*args, **kwargs)

            return update_wrapper(wrapper, view)

        info = self.model._meta.app_label, self.model._meta.model_name

        urls = [
            url(r"^(.+)/zwroc/$", wrap(self.zwroc_view), name="%s_%s_zwroc" % info),
        ]

        super_urls = super().get_urls()

        return urls + super_urls

    zwroc_view_template = (
        "admin/zglos_publikacje/zgloszenie_publikacji/zwroc_zgloszenie.html"
    )

    @transaction.atomic
    def zwroc_view(self, request, id, form_url="", extra_context=None):
        opts = Zgloszenie_Publikacji._meta
        obj = Zgloszenie_Publikacji.objects.get(pk=id)
        form_class = ZwrocEmailForm

        add = False

        if request.method == "POST":
            form = form_class(request.POST, request.FILES, instance=obj)
            form_validated = form.is_valid()
            if form_validated:
                new_object = self.save_form(request, form, change=True)
                new_object.status = Zgloszenie_Publikacji.Statusy.WYMAGA_ZMIAN
                new_object.kod_do_edycji = uuid.uuid4()
                self.save_model(request, new_object, form, not add)
                change_message = f"Rekord został zwrócony, przyczyna {form.cleaned_data['przyczyna_zwrotu']}"
                self.log_change(request, new_object, change_message)

                def _():
                    sent_okay = None
                    try:
                        send_templated_mail(
                            template_name="zwroc_zgloszenie",
                            from_email=getattr(
                                settings, "DEFAULT_FROM_EMAIL", "webmaster@localhost"
                            ),
                            headers={"reply-to": request.user.email},
                            recipient_list=[obj.email],
                            context={
                                "object": obj,
                                "site_url": request.get_host(),
                            },
                        )
                        sent_okay = True
                    except Exception as e:
                        capture_exception(e)

                        messages.add_message(
                            request,
                            messages.WARNING,
                            f"Z uwagi na błąd wysyłania komunikatu z powiadomieniem, użytkownik {obj.email} "
                            "nie został powiadomiony o dodaniu zgłoszenia. Prosimy wysłać e-mail ręcznie. ",
                        )

                    if sent_okay:
                        messages.add_message(
                            request,
                            messages.INFO,
                            f"Użytkownik {obj.email} został powiadomiony o zwróceniu zgłoszenia przez e-mail.",
                        )

                transaction.on_commit(_)
                return HttpResponseRedirect("../..")

        preserved_filters = self.get_preserved_filters(request)
        form_url = add_preserved_filters(
            {"preserved_filters": preserved_filters, "opts": opts}, form_url
        )

        fieldsets = [(None, {"fields": ["przyczyna_zwrotu"]})]

        adminForm = helpers.AdminForm(form_class(), fieldsets, {}, model_admin=self)

        context = {
            "title": "Zwróć zgłoszenie %s" % obj,
            "has_change_permission": True,
            "has_view_permission": True,
            "has_editable_inline_admin_formsets": False,
            "has_delete_permission": False,
            "has_add_permission": False,
            "form_url": form_url,
            "adminform": adminForm,
            "opts": opts,
            "app_label": opts.app_label,
            "original": obj,
            "add": add,
            "change": True,
            "is_popup": False,
            "save_as": False,
            "show_save": False,
            "show_save_and_add_another": False,
            "show_save_and_continue": True,
        }
        context.update(extra_context or {})

        return render(request, self.zwroc_view_template, context)

    def wydzial_pierwszego_autora(self, obj):
        try:
            return obj.zgloszenie_publikacji_autor_set.first().jednostka.wydzial.nazwa
        except BaseException:
            pass

    wydzial_pierwszego_autora.short_description = "Wydział pierwszego autora"



# -*- coding: utf-8 -*-


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0065_merge'),
    ]

    operations = [
        migrations.AddField(
            model_name='wydawnictwo_zwarte',
            name='calkowita_liczba_autorow',
            field=models.PositiveIntegerField(help_text=b'Je\xc5\xbceli dodajesz monografi\xc4\x99, wpisz tutaj ca\xc5\x82kowit\xc4\x85 liczb\xc4\x99\n        autor\xc3\xb3w monografii. Ta informacja zostanie u\xc5\xbcyta w eksporcie danych do PBN.', null=True, blank=True),
        ),
    ]



# Generated by Django 3.0.11 on 2020-12-25 23:59

from django.db import migrations

from bpp.migration_util import load_custom_sql


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0228_merge_20201225_1931"),
    ]

    operations = [
        migrations.RunPython(
            lambda *args, **kw: load_custom_sql("0229_nowe_sumy_status_korekty")
        )
    ]



# -*- encoding: utf-8 -*-


"""Wspólne procedury dla raportu jednostek oraz autorów.
"""

import itertools
import sys
import urllib.request, urllib.parse, urllib.error
from decimal import Decimal

import bleach
from django.db.models.query_utils import Q

from bpp.models import Charakter_Formalny, Jezyk, Typ_Odpowiedzialnosci
from bpp.models.cache import Autorzy, Rekord
from bpp.models.system import Typ_KBN
from bpp.models.abstract import ILOSC_ZNAKOW_NA_ARKUSZ
from django.core.exceptions import ObjectDoesNotExist
from django.http.response import HttpResponse
from django.template import loader
from collections import OrderedDict as SortedDict
from django.utils.safestring import mark_safe
from django.views.generic.detail import DetailView
from django_tables2 import A, Column, Table


def sumif(table):
    return sum(x.impact_factor for x in table.data)


def sumpk(table):
    return sum(x.punkty_kbn for x in table.data)


def sumif_kc(table):
    return sum(x.kc_impact_factor or x.impact_factor for x in table.data)


def sumpk_kc(table):
    return sum(x.kc_punkty_kbn or x.punkty_kbn for x in table.data)


class SumyImpactKbnMixin(Table):
    impact_factor = Column(
        "IF", footer=sumif_kc, attrs={"td": {"align": "right"}}, orderable=False,
    )
    punkty_kbn = Column(
        "PK (MNiSzW) x",
        footer=sumpk_kc,
        attrs={"td": {"align": "right"}},
        orderable=False,
    )


class TypowaTabelaMixin(Table):
    lp = Column("Lp.", A("id"), empty_values=(), orderable=False)
    tytul_oryginalny = Column("Tytuł")
    jezyk = Column("Język", A("jezyk.skrot"), orderable=False)
    autorzy = Column(
        "Autor (autorzy)",
        A("opis_bibliograficzny_zapisani_autorzy_cache"),
        orderable=False,
    )

    def __init__(self, *args, **kw):
        super(TypowaTabelaMixin, self).__init__(*args, **kw)
        self.counter = 0

    def render_lp(self, record):
        self.counter += 1
        return "%d." % self.counter

    #
    # def render_autorzy(self, record):
    #     return record.opis_bibliograficzny_zapisani_autorzy_cache

    def render_tytul_oryginalny(self, record):
        return mark_safe(record.tytul_oryginalny)


class Tabela_Publikacji(TypowaTabelaMixin, SumyImpactKbnMixin, Table):
    class Meta:
        attrs = {"class": "paleblue"}
        template = "raporty/raport_jednostek_autorow_2012/publikacje.html"
        per_page = sys.maxsize
        empty_text = "Brak takich rekordów."
        sequence = (
            "lp",
            "zrodlo",
            "lp_art",
            "autorzy",
            "tytul_oryginalny",
            "rok",
            "szczegoly",
            "punkty_kbn",
        )

    span_rows = 7
    print_sum_impact = False

    zrodlo = Column("Czasopismo", A("zrodlo.nazwa"), orderable=False)
    lp_art = Column("Lp. art.", A("id"), empty_values=(), orderable=False)
    rok = Column("Rok, tom/nr", A("rok"), orderable=False)
    szczegoly = Column("Szczegóły", A("szczegoly"))

    def __init__(self, *args, **kwargs):
        super(Tabela_Publikacji, self).__init__(*args, **kwargs)
        self.zrodlo_counter = itertools.count(1)
        self.output_zrodlo = False
        self.old_zrodlo = None

    def render_lp_art(self, record):
        ret = "%s.%s." % (self.counter, next(self.zrodlo_counter))
        return ret

    def render_lp(self, record):
        # record.zrodlo moze byc None
        try:
            new_zrodlo = record.zrodlo.nazwa
        except ObjectDoesNotExist:
            new_zrodlo = ""

        if self.old_zrodlo != new_zrodlo:
            self.counter += 1
            self.zrodlo_counter = itertools.count(1)
            self.old_zrodlo = new_zrodlo
            self.nowe_zrodlo_w_wierszu = True
        else:
            self.nowe_zrodlo_w_wierszu = False

        if self.nowe_zrodlo_w_wierszu:
            return "%d." % self.counter
        return ""

    def render_zrodlo(self, record):
        if self.nowe_zrodlo_w_wierszu:
            return record.zrodlo.nazwa

        return ""

    def render_szczegoly(self, record):
        return record.szczegoly

    def render_rok(self, record):
        if str(record.rok) in record.informacje:
            return record.informacje
        return "%s, %s" % (record.rok, record.informacje)


class Tabela_Publikacji_Z_Impactem(Tabela_Publikacji):
    class Meta:
        attrs = {"class": "paleblue"}
        template = "raporty/raport_jednostek_autorow_2012/publikacje.html"
        per_page = sys.maxsize
        empty_text = "Brak takich rekordów."
        sequence = (
            "lp",
            "zrodlo",
            "lp_art",
            "autorzy",
            "tytul_oryginalny",
            "rok",
            "szczegoly",
            "impact_factor",
            "punkty_kbn",
        )

    span_rows = 7
    print_sum_impact = True


class Tabela_Konferencji_Miedzynarodowej(TypowaTabelaMixin, SumyImpactKbnMixin, Table):
    class Meta:
        attrs = {"class": "paleblue"}
        template = "raporty/raport_jednostek_autorow_2012/publikacje.html"
        per_page = sys.maxsize
        empty_text = "Brak takich rekordów."
        sequence = (
            "lp",
            "zrodlo",
            "lp_art",
            "autorzy",
            "tytul_oryginalny",
            "rok",
            "zakres",
            "punkty_kbn",
        )

    span_rows = 7
    print_sum_impact = False

    zrodlo = Column("Tytuł materiału konf.", A("informacje"), orderable=False)
    lp_art = Column("Lp. ref.", A("id"), empty_values=(), orderable=False)
    rok = Column("Rok")
    zakres = Column("Zakres stron ref.", A("szczegoly"), orderable=False)

    def __init__(self, *args, **kwargs):
        super(Tabela_Konferencji_Miedzynarodowej, self).__init__(*args, **kwargs)
        self.zrodlo_counter = itertools.count(1)
        self.output_zrodlo = False
        self.old_zrodlo = None

    def render_lp_art(self, record):
        ret = "%s.%s." % (self.counter, next(self.zrodlo_counter))
        return ret

    def render_lp(self, record):
        # record.zrodlo moze byc None
        try:
            new_zrodlo = record.informacje
        except ObjectDoesNotExist:
            new_zrodlo = ""

        if self.old_zrodlo != new_zrodlo:
            self.counter += 1
            self.zrodlo_counter = itertools.count(1)
            self.old_zrodlo = new_zrodlo
            self.nowe_zrodlo_w_wierszu = True
        else:
            self.nowe_zrodlo_w_wierszu = False

        if self.nowe_zrodlo_w_wierszu:
            return "%d." % self.counter
        return ""

    def render_zrodlo(self, record):
        if self.nowe_zrodlo_w_wierszu:
            return record.informacje.strip("W: ")

        return ""

    def render_zakres(self, record):
        buf = record.szczegoly
        # if record.uwagi:
        #     buf += u", " + record.uwagi
        #
        buf = buf.strip("WoS").strip()
        return buf


class Tabela_Monografii(TypowaTabelaMixin, SumyImpactKbnMixin, Table):
    class Meta:
        attrs = {"class": "paleblue"}
        template = "raporty/raport_jednostek_autorow_2012/monografie.html"
        per_page = sys.maxsize
        empty_text = "Brak takich rekordów."
        sequence = (
            "lp",
            "autorzy",
            "wydawca",
            "tytul_oryginalny",
            "jezyk",
            "rok",
            "szczegoly",
            "punkty_kbn",
        )

    szczegoly = Column("Szczegóły", A("szczegoly"))

    wydawca = Column("Wydawca", A("wydawnictwo"), orderable=False)
    rok = Column("Rok", orderable=False)

    def render_szczegoly(self, record):
        buf = record.szczegoly
        if record.uwagi:
            buf += ", " + record.uwagi
        return buf


class Tabela_Redakcji_Naukowej(TypowaTabelaMixin, SumyImpactKbnMixin, Table):
    class Meta:
        attrs = {"class": "paleblue"}
        template = "raporty/raport_jednostek_autorow_2012/monografie.html"
        per_page = sys.maxsize
        empty_text = "Brak takich rekordów."
        sequence = (
            "lp",
            "redaktorzy",
            "wydawca",
            "tytul_oryginalny",
            "jezyk",
            "rok",
            "szczegoly",
            "punkty_kbn",
        )

    szczegoly = Column("Szczegóły", A("szczegoly"))

    wydawca = Column("Wydawca", A("wydawnictwo"), orderable=False)
    rok = Column("Rok", orderable=False)

    redaktorzy = Column(
        "Redaktor (redaktorzy)",
        A("opis_bibliograficzny_autorzy_cache"),
        orderable=False,
    )

    def render_redaktorzy(self, record):
        klass = record.original.autorzy.through
        to = Typ_Odpowiedzialnosci.objects.get(skrot="red.")
        reds = klass.objects.filter(rekord=record.original, typ_odpowiedzialnosci=to)
        return ", ".join([x.zapisany_jako for x in reds])


def split_red(s, want, if_no_result=None):
    seps = [
        "Pod. red.",
        "Pod red.",
        "Red. nauk.",
        "red.",
        "Pod redakcją",
        "pod redakcją",
        "Red.",
        "Ed. by",
    ]

    for sep in seps:
        if s.find(sep) > 0:
            x = s.split(sep)
            if len(x) > want:
                return x[want]

    if if_no_result:
        return if_no_result

    return s


def get_tytul_monografii(record):
    orig = record.original

    if hasattr(orig, "wydawnictwo_nadrzedne"):
        wn = orig.wydawnictwo_nadrzedne
        if wn is not None:
            return wn.tytul_oryginalny

    return split_red(record.informacje, 0, if_no_result="").replace("W: ", "")


class Tabela_Rozdzialu_Monografii(TypowaTabelaMixin, SumyImpactKbnMixin, Table):
    class Meta:
        attrs = {"class": "paleblue"}
        template = "raporty/raport_jednostek_autorow_2012/rozdzialy_monografii.html"
        per_page = sys.maxsize
        empty_text = "Brak takich rekordów."
        sequence = (
            "lp",
            "tytul",
            "wydawca",
            "lp_rozdzialu",
            "autorzy",
            "tytul_rozdzialu",
            "jezyk",
            "rok",
            "szczegoly",
            # 'impact_factor',
            # redaktor monografii
            "punkty_kbn",
        )

    tytul = Column("Tytuł monografii", A("pk"))
    tytul_rozdzialu = Column("Tytuł rozdziału", A("tytul_oryginalny"))

    wydawca = Column("Wydawca", A("wydawnictwo"), orderable=False)
    # impact_factor  = Column("Redaktor monografii")
    rok = Column("Rok", orderable=False)
    szczegoly = Column("Szczegóły", A("szczegoly"))
    lp_rozdzialu = Column("Lp. rozdziału", A("szczegoly"))

    def __init__(self, *args, **kwargs):
        super(Tabela_Rozdzialu_Monografii, self).__init__(*args, **kwargs)

        self.counter = 0
        self.old_tm = None

    def render_lp(self, record):
        tm = get_tytul_monografii(record)

        if self.old_tm != tm:
            self.counter += 1
            self.tm_counter = itertools.count(1)
            self.old_tm = tm
            self.nowy_tytul_w_wierszu = True
        else:
            self.nowy_tytul_w_wierszu = False

        if self.nowy_tytul_w_wierszu:
            return "%d." % self.counter
        return ""

    def render_lp_rozdzialu(self, record):
        ret = "%s.%s." % (self.counter, next(self.tm_counter))
        return ret

    def render_tytul(self, record):
        if self.nowy_tytul_w_wierszu:
            return get_tytul_monografii(record)
        return ""

    def render_wydawca(self, record):
        if self.nowy_tytul_w_wierszu:
            return record.wydawnictwo
        return ""

    def render_tytul_rozdzialu(self, record):
        return record.tytul_oryginalny

    def render_szczegoly(self, record):
        buf = record.szczegoly
        if record.uwagi:
            buf += ", " + record.uwagi
        return buf

        # def render_impact_factor(self, record): # redaktor monografii
        #     orig = record.original
        #
        #     if hasattr(orig, 'wydawnictwo_nadrzedne'):
        #         wn = orig.wydawnictwo_nadrzedne
        #         if wn is not None:
        #             redaktorzy = Wydawnictwo_Zwarte_Autor.objects.filter(
        #                 rekord=wn,
        #                 typ_odpowiedzialnosci=Typ_Odpowiedzialnosci.objects.get(
        #                     skrot='red.'))
        #
        #             return u", ".join([unicode(red) for red in redaktorzy])
        #
        #     return split_red(record.informacje, 1, "").strip(" .")


def wiele(model, *args):
    return [model.objects.get(skrot=x) for x in args]


def charaktery(*args):
    return wiele(Charakter_Formalny, *args)


def jezyki(*args):
    return wiele(Jezyk, *args)


def jezyki_obce():
    return jezyki("ang.", "niem.", "fr.", "hiszp.", "ros.", "wł.")


WSZYSTKIE_TABELE = SortedDict(
    [
        ("1_1", Tabela_Publikacji_Z_Impactem),
        ("1_2", Tabela_Publikacji),
        ("1_3", Tabela_Publikacji),
        ("1_4", Tabela_Publikacji),
        ("1_5", Tabela_Konferencji_Miedzynarodowej),
        ("2_1", Tabela_Monografii),
        ("2_2", Tabela_Rozdzialu_Monografii),
        ("2_3", Tabela_Redakcji_Naukowej),
    ]
)


def get_extra_kw_for_jednostka(jednostka, typ_autora_skrot):
    return {
        "autorzy__jednostka_id": jednostka.pk,
        "autorzy__typ_odpowiedzialnosci__skrot": typ_autora_skrot,
    }


def get_extra_kw_for_autor(autor, typ_autora_skrot):
    return {
        "autorzy__autor_id": autor.pk,
        "autorzy__typ_odpowiedzialnosci__skrot": typ_autora_skrot,
    }


def raport_common_tabela(key, base_query, jednostka=None, autor=None):
    """Modyfikuje base_query według zadanego klucza.
    """
    """Modyfikuje queryset base_query według parametrów dla zadanego
    raportu (key)
    """

    assert not (jednostka != None and autor != None), "albo autor, albo jednostka"

    if key == "1_1":
        return (
            base_query.filter(impact_factor__gt=0, punktacja_wewnetrzna=0)
            .exclude(adnotacje__icontains="wos")
            .exclude(adnotacje__icontains="erih")
            .exclude(typ_kbn=Typ_KBN.objects.get(skrot="PW"))
            .order_by("zrodlo__nazwa")
        )

    elif key == "1_2":
        # (Charakter formalny= AC OR L OR Supl) AND IF=0 AND PK>0 AND NOT
        # [(Charakter formalny= AC OR L OR Supl) AND IF=0 PK>0 AND l. znaków> 20000]

        return (
            base_query.filter(
                charakter_formalny__skrot__in=["AC", "L", "Supl"],
                impact_factor=0,
                punkty_kbn__gt=0,
            )
            .exclude(adnotacje__icontains="wos")
            .exclude(adnotacje__icontains="erih")
            .exclude(typ_kbn=Typ_KBN.objects.get(skrot="PW"))
        )

    elif key == "1_3":
        return base_query.filter(adnotacje__icontains="erih", punkty_kbn__gt=0)

    elif key == "1_4":
        # 1.4 Recenzowana publikacja naukowa w języku innym niż polski w
        # zagranicznym czasopiśmie naukowym spoza list A,B,C, o objętości co
        # najmniej 0,5 arkusza

        # (Charakter formalny= AC OR L OR Supl) AND IF=0 AND PK>0 AND l. znaków> 20000

        return base_query.filter(
            liczba_znakow_wydawniczych__gte=ILOSC_ZNAKOW_NA_ARKUSZ / 2,
            charakter_formalny__skrot__in=["AC", "L", "Supl"],
            impact_factor=0,
            punkty_kbn__gt=0,
        ).exclude(jezyk=Jezyk.objects.get(skrot="pol."))

    elif key == "1_5":
        return base_query.filter(adnotacje__icontains="wos", punkty_kbn__gt=0)

    elif key == "2_1":
        extra_kw = {}
        # Jeżeli podana jest jednostka jako parametr, no to wówczas wyszukujemy
        # autorów tylko z tej jednostki o zadanym typie:
        if jednostka is not None:
            extra_kw = get_extra_kw_for_jednostka(jednostka, "aut.")
        if autor is not None:
            extra_kw = get_extra_kw_for_autor(autor, "aut.")
        return base_query.filter(
            charakter_formalny__in=[
                Charakter_Formalny.objects.get(skrot="KSZ"),
                Charakter_Formalny.objects.get(skrot="KSP"),
                Charakter_Formalny.objects.get(skrot="KS"),
            ],
            punkty_kbn__gt=0,
            **extra_kw
        )

    elif key == "2_2":
        extra_kw = {}
        # Jeżeli podana jest jednostka jako parametr, no to wówczas wyszukujemy
        # autorów tylko z tej jednostki o zadanym typie:
        if jednostka is not None:
            extra_kw = get_extra_kw_for_jednostka(jednostka, "aut.")
        if autor is not None:
            extra_kw = get_extra_kw_for_autor(autor, "aut.")
        return base_query.filter(
            charakter_formalny=Charakter_Formalny.objects.get(skrot="ROZ"),
            punkty_kbn__gt=0,
            **extra_kw
        ).order_by("informacje", "tytul_oryginalny")

    elif key == "2_3":
        extra_kw = {}
        # Jeżeli podana jest jednostka jako parametr, no to wówczas wyszukujemy
        # autorów tylko z tej jednostki o zadanym typie:
        if jednostka is not None:
            extra_kw = get_extra_kw_for_jednostka(jednostka, "red.")
        if autor is not None:
            extra_kw = get_extra_kw_for_autor(autor, "red.")
        return base_query.filter(
            charakter_formalny__in=[
                Charakter_Formalny.objects.get(skrot="KSZ"),
                Charakter_Formalny.objects.get(skrot="KSP"),
                Charakter_Formalny.objects.get(skrot="KS"),
            ],
            punkty_kbn__gt=0,
            **extra_kw
        )


def raport_jednostek_tabela(key, base_query, jednostka):
    return raport_common_tabela(key, base_query, jednostka=jednostka)


def raport_autorow_tabela(key, base_query, autor):
    return raport_common_tabela(key, base_query, autor=autor)


def _get_base_query(fun, param, rok_min, rok_max):
    base = fun(param)
    if rok_min == rok_max:
        return base.filter(rok=rok_min)

    assert rok_min < rok_max
    return base.filter(rok__gte=rok_min, rok__lte=rok_max)


def get_base_query_jednostka(jednostka, *args, **kw):
    return _get_base_query(Rekord.objects.prace_jednostki, jednostka, *args, **kw)


def get_base_query_autor(autor, *args, **kw):
    return _get_base_query(
        Rekord.objects.prace_autora_z_afiliowanych_jednostek, autor, *args, **kw
    )


MSW_ALLOWED_TAGS = [
    "abbr",
    "acronym",
    "b",
    "blockquote",
    "code",
    "em",
    "i",
    "li",
    "ol",
    "strong",
    "ul",
    "center",
    "h1",
    "h2",
    "h3",
    "h4",
    "h5",
    "h6",
    "table",
    "tr",
    "td",
    "th",
    "div",
    "thead",
    "tbody",
    "body",
    "head",
    "meta",
    "html",
]

MSW_ALLOWED_ATTRIBUTES = {
    "a": ["href", "title"],
    "td": ["colspan", "rowspan", "align", "valign"],
    "abbr": ["title"],
    "acronym": ["title"],
    "meta": ["charset"],
}


class MSWordFromTemplateResponse(HttpResponse):
    def __init__(self, request, context, template_name, visible_name, *args, **kwargs):
        super(HttpResponse, self).__init__(*args, **kwargs)
        self["Content-type"] = "application/msword"
        self["Content-disposition"] = "attachment; filename=%s" % visible_name.encode(
            "utf-8"
        )  # urllib.quote(visible_name.encode("utf-8"))
        c = loader.render_to_string(template_name, context, request=request)
        c = bleach.clean(
            c, tags=MSW_ALLOWED_TAGS, attributes=MSW_ALLOWED_ATTRIBUTES, strip=True
        )
        c = c.replace("<table>", "<table border=1 cellspacing=0>")
        self.content = (
            '<html><head><meta charset="utf-8"></head><body>' + c + "</body></html>"
        )
        self["Content-length"] = len(self.content)


class Raport2012CommonView(DetailView):
    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        context = self.get_context_data(object=self.object)
        if context["output"] == "msw":
            zakres_lat = "%s" % context["rok_min"]
            if context["rok_min"] != context["rok_max"]:
                zakres_lat += "-%s" % context["rok_max"]
            nazwa_pliku = "raport-%s-%s.doc" % (
                self.get_short_object_name().replace(".", "").replace(" ", ""),
                zakres_lat,
            )
            return MSWordFromTemplateResponse(
                self.request,
                context,
                "raporty/raport_jednostek_autorow_2012/raport_common.html",
                nazwa_pliku,
            )
        return self.render_to_response(context)



# Generated by Django 2.2.10 on 2020-02-29 15:44

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0197_auto_20200223_2142"),
    ]

    operations = [
        migrations.AddField(
            model_name="wydawca",
            name="alias_dla",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="bpp.Wydawca",
            ),
        ),
        migrations.AlterField(
            model_name="patent",
            name="strony",
            field=models.CharField(
                blank=True,
                help_text='Jeżeli uzupełnione, to pole będzie eksportowane do\n        danych PBN. Jeżeli puste, informacja ta będzie ekstrahowana z\n        pola "Szczegóły" w chwili generowania eksportu PBN. Aby uniknąć\n        sytuacji, gdy wskutek błędnego wprowadzenia tekstu do pola\n        "Szczegóły" informacja ta nie będzie mogła być wyekstrahowana\n        z tego pola, kliknij przycisk "Uzupełnij", aby spowodować uzupełnienie\n        tego pola na podstawie pola "Szczegóły".\n        ',
                max_length=250,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="patent",
            name="tom",
            field=models.CharField(
                blank=True,
                help_text="Jeżeli uzupełnione, to pole będzie eksportowane do\n        danych PBN. Jeżeli puste, informacja ta będzie ekstrahowana z\n        pola 'Informacje'. Kliknięcie przycisku \"Uzupełnij\" powoduje\n        również automatyczne wypełnienie tego pola, o ile do formularza\n        zostały wprowadzone odpowiednie informacje. ",
                max_length=50,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="patent_autor",
            name="afiliuje",
            field=models.BooleanField(
                default=True,
                help_text="Afiliuje\n    się do jednostki podanej w przypisaniu",
            ),
        ),
        migrations.AlterField(
            model_name="patent_autor",
            name="zatrudniony",
            field=models.BooleanField(
                default=False,
                help_text="Pracownik\n    jednostki podanej w przypisaniu",
            ),
        ),
        migrations.AlterField(
            model_name="praca_doktorska",
            name="liczba_cytowan",
            field=models.PositiveIntegerField(
                blank=True,
                help_text="Wartość aktualizowana jest automatycznie raz na kilka dni w przypadku\n        skonfigurowania dostępu do API WOS AMR (przez obiekt 'Uczelnia'). Możesz również\n        czaktualizować tą wartość ręcznie, naciskając przycisk. ",
                null=True,
                verbose_name="Liczba cytowań",
            ),
        ),
        migrations.AlterField(
            model_name="praca_doktorska",
            name="strony",
            field=models.CharField(
                blank=True,
                help_text='Jeżeli uzupełnione, to pole będzie eksportowane do\n        danych PBN. Jeżeli puste, informacja ta będzie ekstrahowana z\n        pola "Szczegóły" w chwili generowania eksportu PBN. Aby uniknąć\n        sytuacji, gdy wskutek błędnego wprowadzenia tekstu do pola\n        "Szczegóły" informacja ta nie będzie mogła być wyekstrahowana\n        z tego pola, kliknij przycisk "Uzupełnij", aby spowodować uzupełnienie\n        tego pola na podstawie pola "Szczegóły".\n        ',
                max_length=250,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="praca_doktorska",
            name="tom",
            field=models.CharField(
                blank=True,
                help_text="Jeżeli uzupełnione, to pole będzie eksportowane do\n        danych PBN. Jeżeli puste, informacja ta będzie ekstrahowana z\n        pola 'Informacje'. Kliknięcie przycisku \"Uzupełnij\" powoduje\n        również automatyczne wypełnienie tego pola, o ile do formularza\n        zostały wprowadzone odpowiednie informacje. ",
                max_length=50,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="praca_habilitacyjna",
            name="liczba_cytowan",
            field=models.PositiveIntegerField(
                blank=True,
                help_text="Wartość aktualizowana jest automatycznie raz na kilka dni w przypadku\n        skonfigurowania dostępu do API WOS AMR (przez obiekt 'Uczelnia'). Możesz również\n        czaktualizować tą wartość ręcznie, naciskając przycisk. ",
                null=True,
                verbose_name="Liczba cytowań",
            ),
        ),
        migrations.AlterField(
            model_name="praca_habilitacyjna",
            name="strony",
            field=models.CharField(
                blank=True,
                help_text='Jeżeli uzupełnione, to pole będzie eksportowane do\n        danych PBN. Jeżeli puste, informacja ta będzie ekstrahowana z\n        pola "Szczegóły" w chwili generowania eksportu PBN. Aby uniknąć\n        sytuacji, gdy wskutek błędnego wprowadzenia tekstu do pola\n        "Szczegóły" informacja ta nie będzie mogła być wyekstrahowana\n        z tego pola, kliknij przycisk "Uzupełnij", aby spowodować uzupełnienie\n        tego pola na podstawie pola "Szczegóły".\n        ',
                max_length=250,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="praca_habilitacyjna",
            name="tom",
            field=models.CharField(
                blank=True,
                help_text="Jeżeli uzupełnione, to pole będzie eksportowane do\n        danych PBN. Jeżeli puste, informacja ta będzie ekstrahowana z\n        pola 'Informacje'. Kliknięcie przycisku \"Uzupełnij\" powoduje\n        również automatyczne wypełnienie tego pola, o ile do formularza\n        zostały wprowadzone odpowiednie informacje. ",
                max_length=50,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="wydawnictwo_ciagle",
            name="liczba_cytowan",
            field=models.PositiveIntegerField(
                blank=True,
                help_text="Wartość aktualizowana jest automatycznie raz na kilka dni w przypadku\n        skonfigurowania dostępu do API WOS AMR (przez obiekt 'Uczelnia'). Możesz również\n        czaktualizować tą wartość ręcznie, naciskając przycisk. ",
                null=True,
                verbose_name="Liczba cytowań",
            ),
        ),
        migrations.AlterField(
            model_name="wydawnictwo_ciagle",
            name="nr_zeszytu",
            field=models.CharField(
                blank=True,
                help_text="Jeżeli uzupełnione, to pole będzie eksportowane do\n        danych PBN. Jeżeli puste, informacja ta będzie ekstrahowana z\n        pola 'Informacje'. Kliknięcie przycisku \"Uzupełnij\" powoduje\n        również automatyczne wypełnienie tego pola, o ile do formularza\n        zostały wprowadzone odpowiednie informacje. ",
                max_length=50,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="wydawnictwo_ciagle",
            name="strony",
            field=models.CharField(
                blank=True,
                help_text='Jeżeli uzupełnione, to pole będzie eksportowane do\n        danych PBN. Jeżeli puste, informacja ta będzie ekstrahowana z\n        pola "Szczegóły" w chwili generowania eksportu PBN. Aby uniknąć\n        sytuacji, gdy wskutek błędnego wprowadzenia tekstu do pola\n        "Szczegóły" informacja ta nie będzie mogła być wyekstrahowana\n        z tego pola, kliknij przycisk "Uzupełnij", aby spowodować uzupełnienie\n        tego pola na podstawie pola "Szczegóły".\n        ',
                max_length=250,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="wydawnictwo_ciagle",
            name="tom",
            field=models.CharField(
                blank=True,
                help_text="Jeżeli uzupełnione, to pole będzie eksportowane do\n        danych PBN. Jeżeli puste, informacja ta będzie ekstrahowana z\n        pola 'Informacje'. Kliknięcie przycisku \"Uzupełnij\" powoduje\n        również automatyczne wypełnienie tego pola, o ile do formularza\n        zostały wprowadzone odpowiednie informacje. ",
                max_length=50,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="wydawnictwo_ciagle_autor",
            name="afiliuje",
            field=models.BooleanField(
                default=True,
                help_text="Afiliuje\n    się do jednostki podanej w przypisaniu",
            ),
        ),
        migrations.AlterField(
            model_name="wydawnictwo_ciagle_autor",
            name="zatrudniony",
            field=models.BooleanField(
                default=False,
                help_text="Pracownik\n    jednostki podanej w przypisaniu",
            ),
        ),
        migrations.AlterField(
            model_name="wydawnictwo_zwarte",
            name="liczba_cytowan",
            field=models.PositiveIntegerField(
                blank=True,
                help_text="Wartość aktualizowana jest automatycznie raz na kilka dni w przypadku\n        skonfigurowania dostępu do API WOS AMR (przez obiekt 'Uczelnia'). Możesz również\n        czaktualizować tą wartość ręcznie, naciskając przycisk. ",
                null=True,
                verbose_name="Liczba cytowań",
            ),
        ),
        migrations.AlterField(
            model_name="wydawnictwo_zwarte",
            name="strony",
            field=models.CharField(
                blank=True,
                help_text='Jeżeli uzupełnione, to pole będzie eksportowane do\n        danych PBN. Jeżeli puste, informacja ta będzie ekstrahowana z\n        pola "Szczegóły" w chwili generowania eksportu PBN. Aby uniknąć\n        sytuacji, gdy wskutek błędnego wprowadzenia tekstu do pola\n        "Szczegóły" informacja ta nie będzie mogła być wyekstrahowana\n        z tego pola, kliknij przycisk "Uzupełnij", aby spowodować uzupełnienie\n        tego pola na podstawie pola "Szczegóły".\n        ',
                max_length=250,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="wydawnictwo_zwarte",
            name="tom",
            field=models.CharField(
                blank=True,
                help_text="Jeżeli uzupełnione, to pole będzie eksportowane do\n        danych PBN. Jeżeli puste, informacja ta będzie ekstrahowana z\n        pola 'Informacje'. Kliknięcie przycisku \"Uzupełnij\" powoduje\n        również automatyczne wypełnienie tego pola, o ile do formularza\n        zostały wprowadzone odpowiednie informacje. ",
                max_length=50,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="wydawnictwo_zwarte_autor",
            name="afiliuje",
            field=models.BooleanField(
                default=True,
                help_text="Afiliuje\n    się do jednostki podanej w przypisaniu",
            ),
        ),
        migrations.AlterField(
            model_name="wydawnictwo_zwarte_autor",
            name="zatrudniony",
            field=models.BooleanField(
                default=False,
                help_text="Pracownik\n    jednostki podanej w przypisaniu",
            ),
        ),
    ]



# -*- encoding: utf-8 -*-

from crispy_forms.helper import FormHelper
from crispy_forms_foundation.layout import (
    ButtonHolder,
    Column,
    Fieldset,
    Layout,
    Row,
    Submit,
)
from dal import autocomplete
from django import forms
from django.core.exceptions import ValidationError
from django.core.validators import MaxValueValidator, MinValueValidator
from django.db.models.aggregates import Max, Min

from bpp.models import Uczelnia
from bpp.models.autor import Autor
from bpp.models.cache import Rekord
from bpp.models.struktura import Jednostka, Wydzial
from bpp.util import formdefaults_html_after, formdefaults_html_before, year_last_month


def wez_lata():
    lata = (
        Rekord.objects.all().values_list("rok", flat=True).distinct().order_by("-rok")
    )
    return zip(lata, lata)


wybory = ["wydzial", "jednostka", "autor"]

OUTPUT_FORMATS = [
    ("html", "wyświetl w przeglądarce"),
    ("docx", "Microsoft Word (DOCX)"),
    ("xlsx", "Microsoft Excel (XLSX)"),
]


class BaseRaportForm(forms.Form):
    od_roku = forms.IntegerField(initial=year_last_month)
    do_roku = forms.IntegerField(initial=Uczelnia.objects.do_roku_default)

    OBJ_FIELD = Row(Column("obiekt"))

    _export = forms.ChoiceField(
        label="Format wyjściowy", choices=OUTPUT_FORMATS, required=True
    )

    def clean(self):
        if "od_roku" in self.cleaned_data and "do_roku" in self.cleaned_data:
            if self.cleaned_data["od_roku"] > self.cleaned_data["do_roku"]:
                raise ValidationError(
                    {
                        "od_roku": ValidationError(
                            'Pole musi być większe lub równe jak pole "Do roku".'
                        )
                    }
                )

    def __init__(self, *args, **kwargs):
        super(BaseRaportForm, self).__init__(*args, **kwargs)

        self.helper = FormHelper()
        self.helper.form_class = "custom"
        self.helper.form_action = "."

        self.helper.layout = Layout(
            Fieldset(
                "Wybierz parametry",
                formdefaults_html_before(self),
                self.OBJ_FIELD,
                Row(
                    Column("od_roku", css_class="large-6 medium-6 small-12"),
                    Column("do_roku", css_class="large-6 medium-6 small-12"),
                ),
                Row(Column("_export")),
                Row(Column("tylko_z_jednostek_uczelni")),
                formdefaults_html_after(self),
            ),
            ButtonHolder(
                Submit(
                    "submit",
                    "Pobierz raport",
                    css_id="id_submit",
                    css_class="submit button",
                ),
            ),
        )

        lata = Rekord.objects.all().aggregate(Min("rok"), Max("rok"))
        for field in self["od_roku"], self["do_roku"]:
            if lata["rok__min"] is not None:
                field.field.validators.append(MinValueValidator(lata["rok__min"]))
            if lata["rok__max"] is not None:
                field.field.validators.append(MaxValueValidator(lata["rok__max"]))


class UczelniaRaportForm(BaseRaportForm):
    OBJ_FIELD = ""

    tylko_z_jednostek_uczelni = forms.BooleanField(
        initial=True,
        label="Tylko prace afiliowane",
        help_text="Odznaczenie tego pola uwzględnia w raporcie rekordy "
        "w których autor przypisany jest do jednostki uczelni, ale nie afiliuje na nią",
        required=False,
    )


class WydzialRaportForm(BaseRaportForm):
    obiekt = forms.ModelChoiceField(
        label="Wydział",
        queryset=Wydzial.objects.all(),
        widget=autocomplete.ModelSelect2(
            url="bpp:wydzial-autocomplete", attrs=dict(style="width: 746px;")
        ),
    )

    tylko_z_jednostek_uczelni = forms.BooleanField(
        initial=True,
        label="Tylko prace afiliowane",
        help_text="Odznaczenie tego pola uwzględnia w raporcie rekordy "
        "w których autor przypisany jest do jednostki z wybranego wydziału, ale nie afiliuje na nią",
        required=False,
    )


class JednostkaRaportForm(BaseRaportForm):
    obiekt = forms.ModelChoiceField(
        label="Jednostka",
        queryset=Jednostka.objects.all(),
        widget=autocomplete.ModelSelect2(url="bpp:jednostka-autocomplete"),
    )

    tylko_z_jednostek_uczelni = forms.BooleanField(
        initial=True,
        label="Tylko prace afiliowane",
        help_text="Odznaczenie tego pola uwzględnia w raporcie rekordy "
        "w których autor przypisany jest do wybranej jednostki, ale nie afiliuje na nią",
        required=False,
    )


class AutorRaportForm(BaseRaportForm):
    obiekt = forms.ModelChoiceField(
        label="Autor",
        queryset=Autor.objects.all(),
        widget=autocomplete.ModelSelect2(url="bpp:public-autor-autocomplete"),
    )

    tylko_z_jednostek_uczelni = forms.BooleanField(
        initial=True,
        label="Tylko prace z afiliacją uczelni",
        help_text="Odznaczenie tego pola uwzględnia w raporcie rekordy "
        "w których autor przypisany jest do jednostek "
        "pozauczelnianych (obcych).",
        required=False,
    )



from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.db.models import CASCADE


class Grant(models.Model):
    nazwa_projektu = models.TextField(blank=True, null=True)
    zrodlo_finansowania = models.TextField(blank=True, null=True)
    numer_projektu = models.CharField(max_length=200, unique=True)
    rok = models.PositiveSmallIntegerField(null=True, blank=True)

    class Meta:
        verbose_name = "grant"
        verbose_name_plural = "granty"

    def __str__(self):
        return f"{self.numer_projektu} {self.nazwa_projektu or ''}".strip()


class Grant_Rekordu(models.Model):
    content_type = models.ForeignKey(ContentType, models.CASCADE)
    object_id = models.PositiveIntegerField()
    rekord = GenericForeignKey()
    grant = models.ForeignKey(Grant, models.PROTECT)

    class Meta:
        verbose_name = "grant rekordu"
        verbose_name_plural = "granty rekordu"

        unique_together = [("grant", "content_type", "object_id")]



# -*- encoding: utf-8 -*-

import pytest
from django.conf import settings
from django.core.cache import cache
from django.core.cache.utils import make_template_fragment_key


@pytest.fixture
def remove_key():
    key = make_template_fragment_key('google')
    cache.delete(key)

@pytest.mark.django_db
def test_google_analytics_disabled(remove_key, client):

    orig_DEBUG = settings.DEBUG
    orig_GAPID = settings.GOOGLE_ANALYTICS_PROPERTY_ID

    try:
        settings.DEBUG = True
        res = client.get("/")
        assert b"GoogleAnalyticsObject" not in res.content

        settings.DEBUG = False
        settings.GOOGLE_ANALYTICS_PROPERTY_ID = ""
        res = client.get("/")
        assert b"GoogleAnalyticsObject" not in res.content

    finally:
        settings.DEBUG = orig_DEBUG
        settings.GOOGLE_ANALYTICS_PROPERTY_ID = orig_GAPID


@pytest.mark.django_db
def test_google_analytics_enabled(remove_key, client):
    from django.conf import settings

    orig_DEBUG = settings.DEBUG
    orig_GAPID = settings.GOOGLE_ANALYTICS_PROPERTY_ID

    try:
        settings.DEBUG = False
        settings.GOOGLE_ANALYTICS_PROPERTY_ID = "foobar"

        res = client.get("/")

        assert b"GoogleAnalyticsObject" in res.content

    finally:
        settings.DEBUG = orig_DEBUG
        settings.GOOGLE_ANALYTICS_PROPERTY_ID = orig_GAPID



from urllib.parse import urlencode

import pytest
from django.core.exceptions import ValidationError
from django.urls.base import reverse

from raport_slotow import const
from raport_slotow.views import SESSION_KEY

from django.contrib.contenttypes.models import ContentType

from bpp.const import DO_STYCZNIA_POPRZEDNI_POTEM_OBECNY, NAJWIEKSZY_REKORD
from bpp.models import Uczelnia
from bpp.models.fields import OpcjaWyswietlaniaField
from bpp.tests import browse_praca_url, normalize_html


@pytest.mark.parametrize(
    "attr,s",
    [
        ("pokazuj_punktacje_wewnetrzna", "Punktacja wewnętrzna"),
        ("pokazuj_index_copernicus", "Index Copernicus"),
    ],
)
def test_uczelnia_praca_pokazuj(uczelnia, wydawnictwo_ciagle, attr, s, client):
    url = reverse(
        "bpp:browse_praca",
        args=(
            ContentType.objects.get(app_label="bpp", model="wydawnictwo_ciagle").pk,
            wydawnictwo_ciagle.pk,
        ),
    )

    setattr(uczelnia, attr, True)
    uczelnia.save()
    res = client.get(url, follow=True)
    assert res.status_code == 200
    assert s in res.rendered_content

    setattr(uczelnia, attr, False)
    uczelnia.save()
    res = client.get(url, follow=True)
    assert res.status_code == 200
    assert s not in res.rendered_content


@pytest.mark.parametrize(
    "attr,s",
    [
        ("pokazuj_status_korekty", "Status"),
        ("pokazuj_praca_recenzowana", "Praca recenzowana"),
    ],
)
def test_uczelnia_praca_pokazuj_pozostale(
    uczelnia, wydawnictwo_ciagle, client, admin_client, attr, s
):
    url = reverse(
        "bpp:browse_praca",
        args=(
            ContentType.objects.get(app_label="bpp", model="wydawnictwo_ciagle").pk,
            wydawnictwo_ciagle.pk,
        ),
    )

    setattr(uczelnia, attr, OpcjaWyswietlaniaField.POKAZUJ_ZAWSZE)
    uczelnia.save()

    res = client.get(url, follow=True)
    assert s in res.rendered_content

    res = admin_client.get(url, follow=True)
    assert s in res.rendered_content

    setattr(uczelnia, attr, OpcjaWyswietlaniaField.POKAZUJ_ZALOGOWANYM)
    uczelnia.save()

    res = client.get(url, follow=True)
    assert s not in res.rendered_content

    res = admin_client.get(url, follow=True)
    assert s in res.rendered_content

    setattr(uczelnia, attr, OpcjaWyswietlaniaField.POKAZUJ_NIGDY)
    uczelnia.save()

    res = client.get(url, follow=True)
    assert s not in res.rendered_content

    res = admin_client.get(url, follow=True)
    assert s not in res.rendered_content


lista_stron_raportow = [
    ("pokazuj_ranking_autorow", "ranking-autorow"),
    ("pokazuj_raport_autorow", "nowe_raporty/autor"),
    ("pokazuj_raport_jednostek", "nowe_raporty/jednostka"),
    ("pokazuj_raport_wydzialow", "nowe_raporty/wydzial"),
    ("pokazuj_raport_dla_komisji_centralnej", "raporty/dla-komisji-centralnej"),
]


@pytest.mark.parametrize("attr,s", lista_stron_raportow)
def test_uczelnia_pokazuj_menu_zawsze(uczelnia, attr, s, client, admin_client):
    url = "/"

    setattr(uczelnia, attr, OpcjaWyswietlaniaField.POKAZUJ_ZAWSZE)
    uczelnia.save()

    res = client.get(url, follow=True)
    assert s in res.rendered_content

    res = admin_client.get(url, follow=True)
    assert s in res.rendered_content


@pytest.mark.parametrize("attr,s", lista_stron_raportow)
def test_uczelnia_pokazuj_menu_zalogowani(uczelnia, attr, s, client, admin_client):
    url = "/"

    setattr(uczelnia, attr, OpcjaWyswietlaniaField.POKAZUJ_ZALOGOWANYM)
    uczelnia.save()

    res = client.get(url, follow=True)
    assert s not in res.rendered_content

    res = admin_client.get(url, follow=True)
    assert s in res.rendered_content


@pytest.mark.parametrize("attr,s", lista_stron_raportow)
def test_uczelnia_pokazuj_menu_nigdy(uczelnia, attr, s, client, admin_client):
    url = "/"

    setattr(uczelnia, attr, OpcjaWyswietlaniaField.POKAZUJ_NIGDY)
    uczelnia.save()

    res = client.get(url, follow=True)
    assert s not in res.rendered_content

    res = admin_client.get(url, follow=True)
    assert s not in res.rendered_content


@pytest.mark.django_db
def test_pokazuj_tabele_slotow_na_stronie_rekordu(
    uczelnia,
    admin_client,
    client,
    praca_z_dyscyplina,
):

    url = browse_praca_url(praca_z_dyscyplina)

    S = "Punktacja dyscyplin i sloty"

    uczelnia.pokazuj_tabele_slotow_na_stronie_rekordu = (
        OpcjaWyswietlaniaField.POKAZUJ_ZAWSZE
    )
    uczelnia.save()

    res = client.get(url)
    assert S in res.rendered_content
    res = admin_client.get(url)
    assert S in res.rendered_content

    uczelnia.pokazuj_tabele_slotow_na_stronie_rekordu = (
        OpcjaWyswietlaniaField.POKAZUJ_ZALOGOWANYM
    )
    uczelnia.save()

    res = client.get(url)
    assert S not in res.rendered_content
    res = admin_client.get(url)
    assert S in res.rendered_content

    uczelnia.pokazuj_tabele_slotow_na_stronie_rekordu = (
        OpcjaWyswietlaniaField.POKAZUJ_NIGDY
    )
    uczelnia.save()

    res = client.get(url)
    assert S not in res.rendered_content
    res = admin_client.get(url)
    assert S not in res.rendered_content


@pytest.mark.parametrize(
    "poszukiwany_ciag,atrybut_uczelni",
    [
        ("raport slotów - autor", "pokazuj_raport_slotow_autor"),
        ("raport slotów - uczelnia", "pokazuj_raport_slotow_uczelnia"),
        ("raport slotów - zerowy", "pokazuj_raport_slotow_zerowy"),
    ],
)
@pytest.mark.django_db
def test_pokazuj_raport_slotow_menu_na_glownej(
    uczelnia, admin_client, client, poszukiwany_ciag, atrybut_uczelni, db
):
    url = reverse("bpp:browse_uczelnia", args=(uczelnia.slug,))
    S = poszukiwany_ciag

    setattr(uczelnia, atrybut_uczelni, OpcjaWyswietlaniaField.POKAZUJ_ZALOGOWANYM)
    uczelnia.save()

    res = client.get(url)
    assert S not in normalize_html(res.rendered_content)
    res = admin_client.get(url)
    assert S in normalize_html(res.rendered_content)

    setattr(uczelnia, atrybut_uczelni, OpcjaWyswietlaniaField.POKAZUJ_NIGDY)
    uczelnia.save()

    res = client.get(url)
    assert S not in normalize_html(res.rendered_content)
    res = admin_client.get(url)
    assert S not in normalize_html(res.rendered_content)

    setattr(uczelnia, atrybut_uczelni, OpcjaWyswietlaniaField.POKAZUJ_ZAWSZE)
    uczelnia.save()

    res = client.get(url)
    assert S in normalize_html(res.rendered_content)
    res = admin_client.get(url)
    assert S in normalize_html(res.rendered_content)

    setattr(uczelnia, atrybut_uczelni, OpcjaWyswietlaniaField.POKAZUJ_GDY_W_ZESPOLE)
    uczelnia.save()

    res = client.get(url)
    assert S not in normalize_html(res.rendered_content)
    res = admin_client.get(url)
    assert S in normalize_html(res.rendered_content)


@pytest.mark.parametrize(
    "nazwa_url,atrybut_uczelni,params",
    [
        ("raport_slotow:index", "pokazuj_raport_slotow_autor", {}),
        ("raport_slotow:raport-slotow-zerowy", "pokazuj_raport_slotow_zerowy", {}),
        (
            "raport_slotow:raport",
            "pokazuj_raport_slotow_autor",
            {},
        ),
        (
            "raport_slotow:lista-raport-slotow-uczelnia",
            "pokazuj_raport_slotow_uczelnia",
            {},
        ),
        (
            "raport_slotow:lista-raport-slotow-uczelnia",
            "pokazuj_raport_slotow_uczelnia",
            {"od_roku": 2000, "do_roku": 2000, "maksymalny_slot": 1, "_export": "html"},
        ),
    ],
)
@pytest.mark.django_db
def test_pokazuj_raport_slotow_czy_mozna_kliknac(
    uczelnia, admin_client, client, autor, nazwa_url, atrybut_uczelni, params
):
    url = reverse(nazwa_url)
    if nazwa_url == "raport_slotow:raport":
        dane_raportu = {
            "obiekt": autor.pk,
            "od_roku": 2016,
            "do_roku": 2017,
            "dzialanie": const.DZIALANIE_SLOT,
            "minimalny_pk": 0,
            "slot": 19,
            "_export": "html",
        }

        for c in admin_client, client:
            s = c.session
            s.update({SESSION_KEY: dane_raportu})
            s.save()

    if params:
        url += "?" + urlencode(params)

    setattr(uczelnia, atrybut_uczelni, OpcjaWyswietlaniaField.POKAZUJ_ZALOGOWANYM)
    uczelnia.save()

    res = client.get(url)
    assert res.status_code == 302
    res = admin_client.get(url)
    assert res.status_code == 200

    setattr(uczelnia, atrybut_uczelni, OpcjaWyswietlaniaField.POKAZUJ_NIGDY)
    uczelnia.save()

    res = client.get(url)
    assert res.status_code == 404
    res = admin_client.get(url)
    assert res.status_code == 404

    setattr(uczelnia, atrybut_uczelni, OpcjaWyswietlaniaField.POKAZUJ_GDY_W_ZESPOLE)
    uczelnia.save()

    res = client.get(url)
    assert res.status_code == 302
    res = admin_client.get(url)
    assert res.status_code == 200

    setattr(uczelnia, atrybut_uczelni, OpcjaWyswietlaniaField.POKAZUJ_ZAWSZE)
    uczelnia.save()

    if atrybut_uczelni == "pokazuj_raport_slotow_uczelnia":
        # dla opcji "pokazuj raport slotów uczelnia" login jest zawsze wymagany
        pass
    else:

        res = client.get(url)
        assert res.status_code == 200
        res = admin_client.get(url)
        assert res.status_code == 200


def test_uczelnia_obca_jednostka(uczelnia, jednostka, obca_jednostka):
    uczelnia.obca_jednostka = obca_jednostka
    uczelnia.save()

    uczelnia.obca_jednostka = jednostka
    with pytest.raises(ValidationError):
        uczelnia.save()


def test_uczelnia_ukryte_statusy(uczelnia, przed_korekta, po_korekcie):
    uczelnia.ukryj_status_korekty_set.create(status_korekty=przed_korekta)
    uczelnia.ukryj_status_korekty_set.create(status_korekty=po_korekcie)

    assert przed_korekta.pk in uczelnia.ukryte_statusy("sloty")
    assert po_korekcie.pk in uczelnia.ukryte_statusy("sloty")


def test_uczelnia_do_roku_default(uczelnia, wydawnictwo_zwarte):
    wydawnictwo_zwarte.rok = 3000
    wydawnictwo_zwarte.save()

    uczelnia.metoda_do_roku_formularze = DO_STYCZNIA_POPRZEDNI_POTEM_OBECNY
    uczelnia.save()

    assert Uczelnia.objects.do_roku_default() != 3000

    uczelnia.metoda_do_roku_formularze = NAJWIEKSZY_REKORD
    uczelnia.save()

    assert Uczelnia.objects.do_roku_default() == 3000



# Generated by Django 3.0.11 on 2020-11-28 23:12

from django.db import migrations, models

from bpp.const import CHARAKTER_SLOTY_REFERAT


def zaznacz_referaty(apps, schema_editor):
    Charakter_Formalny = apps.get_model("bpp", "Charakter_Formalny")
    for elem in Charakter_Formalny.objects.all():
        if "referat" in elem.nazwa.lower():
            elem.charakter_sloty = CHARAKTER_SLOTY_REFERAT
            elem.save()


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0224_auto_20201111_1341"),
    ]

    operations = [
        migrations.AlterField(
            model_name="charakter_formalny",
            name="charakter_sloty",
            field=models.PositiveSmallIntegerField(
                blank=True,
                choices=[(1, "Książka"), (2, "Rozdział"), (3, "Referat")],
                default=None,
                help_text="Jak potraktować ten charakter przy kalkulacji slotów dla wydawnictwa zwartego?",
                null=True,
                verbose_name="Charakter dla slotów",
            ),
        ),
        migrations.RunPython(zaznacz_referaty, migrations.RunPython.noop),
    ]



# -*- encoding: utf-8 -*-

__author__ = 'dotz'

import os


def files_or_directory(args, extension=".xlsx"):
    pliki = args
    if os.path.isdir(args[0]):
        pliki = [
            os.path.join(args[0], x) for x in os.listdir(args[0]) if x.endswith(extension)]

    for plik in pliki:
        yield plik



# Generated by Django 3.0.14 on 2021-08-08 23:20

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("pbn_api", "0021_auto_20210808_1204"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="scientist",
            options={
                "ordering": (
                    "from_institution_api",
                    "lastName",
                    "name",
                    "qualifications",
                ),
                "verbose_name": "Osoba w PBN API",
                "verbose_name_plural": "Osoby w PBN API",
            },
        ),
        migrations.AlterUniqueTogether(
            name="publication",
            unique_together={("mongoId", "title", "isbn", "doi", "publicUri")},
        ),
    ]



# -*- encoding: utf-8 -*-


from django.contrib.sitemaps import Sitemap

try:
    from django.core.urlresolvers import reverse
except ImportError:
    from django.urls import reverse

from bpp.models import Jednostka
from bpp.models.autor import Autor
from bpp.models.patent import Patent
from bpp.models.praca_doktorska import Praca_Doktorska
from bpp.models.praca_habilitacyjna import Praca_Habilitacyjna
from bpp.models.struktura import Uczelnia, Wydzial
from bpp.models.wydawnictwo_ciagle import Wydawnictwo_Ciagle
from bpp.models.wydawnictwo_zwarte import Wydawnictwo_Zwarte


class BppSitemap(Sitemap):
    url_obj_field = "pk"

    def items(self):
        return self.klass.objects.all()

    def lastmod(self, obj):
        return obj.ostatnio_zmieniony

    def location(self, obj):
        return reverse(self.url, args=(getattr(obj, self.url_obj_field),))


class JednostkaSitemap(BppSitemap):
    changefreq = "yearly"
    klass = Jednostka
    url = "bpp:browse_jednostka"
    url_obj_field = "slug"


class UczelniaSitemap(BppSitemap):
    changefreq = "yearly"
    klass = Uczelnia
    url = "bpp:browse_uczelnia"
    url_obj_field = "slug"

    def items(self):
        # Funkcja musi zwrócić posortowane wyniki, w przeciwnym wypadku będzie ostrzeżenie
        return super().items().order_by("slug")


class WydzialSitemap(BppSitemap):
    changefreq = "yearly"
    klass = Wydzial
    url = "bpp:browse_wydzial"
    url_obj_field = "slug"

    def items(self):
        return self.klass.objects.filter(widoczny=True)


class AlphabeticBppSitemap(BppSitemap):
    changefreq = "weekly"
    url = "bpp:browse_praca"
    title_field = "tytul_oryginalny"
    litera = None

    def __init__(self, litera=""):
        super(AlphabeticBppSitemap, self).__init__()
        if litera:
            self.litera = litera

    def items(self):
        if not self.litera:
            return list(super(AlphabeticBppSitemap, self).items())
        kw = {self.title_field + "__istartswith": self.litera}
        return list(super(AlphabeticBppSitemap, self).items().filter(**kw))


class AutorSitemap(AlphabeticBppSitemap):
    changefreq = "weekly"
    klass = Autor
    url = "bpp:browse_autor"
    url_obj_field = "slug"
    title_field = "nazwisko"


class PracaBppSitemap(AlphabeticBppSitemap):
    def location(self, obj):
        return reverse(self.url, args=(str(self.klass.__name__).lower(), obj.pk))


class Wydawnictwo_CiagleSitemap(PracaBppSitemap):
    klass = Wydawnictwo_Ciagle


class Wydawnictwo_ZwarteSitemap(PracaBppSitemap):
    klass = Wydawnictwo_Zwarte


class Praca_DoktorskaSitemap(PracaBppSitemap):
    klass = Praca_Doktorska


class Praca_HabilitacyjnaSitemap(PracaBppSitemap):
    klass = Praca_Habilitacyjna


class PatentSitemap(PracaBppSitemap):
    klass = Patent


django_bpp_sitemaps = {
    "jednostka": JednostkaSitemap,
    "uczelnia": UczelniaSitemap,
    "wydzial": WydzialSitemap,
}

for litera in "aąbcćdefghijklłmnńoópqrsśtuvwxyzźż":
    for label, klasa in [
        ("wydawnictwo-zwarte", Wydawnictwo_ZwarteSitemap),
        ("wydawnictwo-ciagle", Wydawnictwo_CiagleSitemap),
        ("praca-doktorska", Praca_DoktorskaSitemap),
        ("praca-habilitacyjna", Praca_HabilitacyjnaSitemap),
        ("autor", AutorSitemap),
        ("patent", PatentSitemap),
    ]:
        django_bpp_sitemaps[label + "-" + litera] = klasa(litera)



from model_bakery import baker


class BPP_Baker(baker.Baker):
    def get_fields(self):
        # Model Bakery nie może podawać tych parametrów dla modeli MPTT
        # https://stackoverflow.com/questions/32267084/model-mommy-breaks-django-mptt

        return [
            field
            for field in super(BPP_Baker, self).get_fields()
            if field.name not in ["lft", "rght"]
        ]



from django.conf import settings
from mptt.forms import TreeNodeChoiceFieldMixin
from mptt.settings import DEFAULT_LEVEL_INDICATOR
from taggit.models import Tag

from .mixins import BppMultiseekVisibilityMixin

from django.contrib.postgres.search import SearchQuery

from django.utils.itercompat import is_iterable

from bpp.models.konferencja import Konferencja
from bpp.models.openaccess import (
    Czas_Udostepnienia_OpenAccess,
    Licencja_OpenAccess,
    Wersja_Tekstu_OpenAccess,
)
from bpp.models.struktura import Wydzial

NULL_VALUE = "(brak wpisanej wartości)"

from django.db.models import Q
from django.db.models.expressions import F
from multiseek import logic
from multiseek.logic import (
    AUTOCOMPLETE,
    DIFFERENT,
    DIFFERENT_ALL,
    DIFFERENT_FEMALE,
    DIFFERENT_NONE,
    EQUAL,
    EQUAL_FEMALE,
    EQUAL_NONE,
    EQUALITY_OPS_ALL,
    EQUALITY_OPS_FEMALE,
    EQUALITY_OPS_MALE,
    EQUALITY_OPS_NONE,
    VALUE_LIST,
    AutocompleteQueryObject,
    BooleanQueryObject,
    DateQueryObject,
    DecimalQueryObject,
    IntegerQueryObject,
    QueryObject,
    RangeQueryObject,
    ReportType,
    StringQueryObject,
    UnknownOperation,
    ValueListQueryObject,
)

from .. import const

from bpp.models import (
    Autor,
    Autorzy,
    Charakter_Formalny,
    Dyscyplina_Naukowa,
    Jednostka,
    Jezyk,
    SlowaKluczoweView,
    Typ_Odpowiedzialnosci,
    Uczelnia,
    Wydawnictwo_Zwarte,
    Zewnetrzna_Baza_Danych,
    ZewnetrzneBazyDanychView,
    Zrodlo,
)
from bpp.models.system import Typ_KBN

UNION = "równy+wspólny"
UNION_FEMALE = "równa+wspólna"
UNION_NONE = "równe+wspólne"
UNION_OPS_ALL = [UNION, UNION_FEMALE, UNION_NONE]


class TytulPracyQueryObject(BppMultiseekVisibilityMixin, StringQueryObject):
    label = "Tytuł pracy"
    field_name = "tytul_oryginalny"

    def real_query(self, value, operation):
        ret = super(StringQueryObject, self).real_query(
            value, operation, validate_operation=False
        )

        if ret is not None:
            return ret

        elif operation in [logic.CONTAINS, logic.NOT_CONTAINS]:
            if not value:
                return Q(pk=F("pk"))

            query = None

            if operation == logic.CONTAINS:
                value = [x.strip() for x in value.split(" ") if x.strip()]
                for elem in value:
                    elem = SearchQuery(elem, config="bpp_nazwy_wlasne")
                    if query is None:
                        query = elem
                        continue
                    query &= elem

            else:
                # Jeżeli "nie zawiera", to nie tokenizuj spacjami
                query = ~SearchQuery(value, config="bpp_nazwy_wlasne")

            ret = Q(search_index=query)

        elif operation in [logic.STARTS_WITH, logic.NOT_STARTS_WITH]:
            ret = Q(**{self.field_name + "__istartswith": value})

            if operation in [logic.NOT_STARTS_WITH]:
                ret = ~ret
        else:
            raise UnknownOperation(operation)

        return ret


class AdnotacjeQueryObject(BppMultiseekVisibilityMixin, StringQueryObject):
    label = "Adnotacje"
    field_name = "adnotacje"
    public = False


class DOIQueryObject(BppMultiseekVisibilityMixin, StringQueryObject):
    label = "DOI"
    field_name = "doi"
    public = False


class InformacjeQueryObject(BppMultiseekVisibilityMixin, StringQueryObject):
    label = "Informacje"
    field_name = "informacje"


class SzczegolyQueryObject(BppMultiseekVisibilityMixin, StringQueryObject):
    label = "Szczegóły"
    field_name = "szczegoly"


class UwagiQueryObject(BppMultiseekVisibilityMixin, StringQueryObject):
    label = "Uwagi"
    field_name = "uwagi"


class SlowaKluczoweQueryObject(BppMultiseekVisibilityMixin, AutocompleteQueryObject):
    type = AUTOCOMPLETE
    ops = [EQUAL_NONE, DIFFERENT_NONE]
    model = Tag
    search_fields = ["name"]
    url = "bpp:public-taggit-tag-autocomplete"
    label = "Słowa kluczowe"
    field_name = "slowa_kluczowe"

    def real_query(self, value, operation):
        if operation in EQUALITY_OPS_ALL:
            ret = Q(
                pk__in=SlowaKluczoweView.objects.filter(tag__name=value).values(
                    "rekord_id"
                )
            )

        else:
            raise UnknownOperation(operation)

        if operation in DIFFERENT_ALL:
            return ~ret

        return ret


class DataUtworzeniaQueryObject(BppMultiseekVisibilityMixin, DateQueryObject):
    label = "Data utworzenia"
    field_name = "utworzono"
    public = False

    def value_for_description(self, value):
        value = self.value_from_web(value)
        if value is None:
            return NULL_VALUE
        if is_iterable(value):
            return f"od {value[0]} do {value[1]}"
        return str(value)


class OstatnioZmieniony(DataUtworzeniaQueryObject):
    label = "Ostatnio zmieniony"
    field_name = "ostatnio_zmieniony"
    public = False


class ForeignKeyDescribeMixin:
    def value_for_description(self, value):
        if value is None:
            return NULL_VALUE

        return self.value_from_web(value) or "[powiązany obiekt został usunięty]"


class NazwiskoIImieQueryObject(
    BppMultiseekVisibilityMixin, ForeignKeyDescribeMixin, AutocompleteQueryObject
):
    label = "Nazwisko i imię"
    type = AUTOCOMPLETE
    ops = [EQUAL_NONE, DIFFERENT_NONE, UNION_NONE]
    model = Autor
    search_fields = ["nazwisko", "imiona"]
    field_name = "autor"
    url = "bpp:public-autor-autocomplete"

    def real_query(self, value, operation):

        if operation in EQUALITY_OPS_ALL:
            ret = Q(autorzy__autor=value)

        elif operation in UNION_OPS_ALL:
            q = Autorzy.objects.filter(autor=value).values("rekord_id")

            ret = Q(pk__in=q)
        else:
            raise UnknownOperation(operation)

        if operation in DIFFERENT_ALL:
            return ~ret

        return ret


class WydawnictwoNadrzedneQueryObject(
    BppMultiseekVisibilityMixin, ForeignKeyDescribeMixin, AutocompleteQueryObject
):
    label = "Wydawnictwo nadrzędne"
    type = AUTOCOMPLETE
    ops = [
        EQUAL_NONE,
        DIFFERENT_NONE,
    ]
    model = Wydawnictwo_Zwarte
    search_fields = [
        "tytul_oryginalny",
    ]
    field_name = "wydawnictwo_nadrzedne"
    url = "bpp:public-wydawnictwo-nadrzedne-autocomplete"

    def real_query(self, value, operation):

        if operation in EQUALITY_OPS_ALL:
            ret = Q(wydawnictwo_nadrzedne=value)

        else:
            raise UnknownOperation(operation)

        if operation in DIFFERENT_ALL:
            return ~ret

        return ret


class ORCIDQueryObject(BppMultiseekVisibilityMixin, StringQueryObject):
    label = "ORCID"
    ops = [EQUAL_NONE, DIFFERENT_NONE]
    field_name = "autorzy__autor__orcid"


class NazwiskoIImieWZakresieKolejnosci(NazwiskoIImieQueryObject):
    ops = [EQUAL, UNION_NONE]
    kolejnosc_gte = None
    kolejnosc_lt = None

    def real_query(self, value, operation):
        if operation in EQUALITY_OPS_ALL:
            ret = Q(
                autorzy__autor=value,
                autorzy__kolejnosc__gte=self.kolejnosc_gte,
                autorzy__kolejnosc__lt=self.kolejnosc_lt,
            )

        elif operation in UNION_OPS_ALL:
            q = Autorzy.objects.filter(
                autor=value,
                kolejnosc__gte=self.kolejnosc_gte,
                kolejnosc__lt=self.kolejnosc_lt,
            ).values("rekord_id")
            ret = Q(pk__in=q)

        else:
            raise UnknownOperation(operation)

        if operation in DIFFERENT_ALL:
            return ~ret

        return ret


class PierwszeNazwiskoIImie(NazwiskoIImieWZakresieKolejnosci):
    kolejnosc_gte = 0
    kolejnosc_lt = 1
    label = "Pierwsze nazwisko i imię"
    field_name = "naz_im_pierwsz"


class OstatnieNazwiskoIImie(NazwiskoIImieWZakresieKolejnosci):
    ops = [
        EQUAL,
    ]
    # bez operatora UNION, bo F('liczba_autorow') nie istnieje, gdy
    # generujemy zapytanie dla niego.
    kolejnosc_gte = F("liczba_autorow") - 1
    kolejnosc_lt = F("liczba_autorow")
    label = "Ostatnie nazwisko i imię"
    public = False
    field_name = "naz_im_ost"


class NazwiskoIImie1do3(NazwiskoIImieWZakresieKolejnosci):
    kolejnosc_gte = 0
    kolejnosc_lt = 3
    label = "Nazwisko i imię (od 1 do 3)"
    public = False
    field_name = "naz_im_1_3"


class NazwiskoIImie1do5(NazwiskoIImieWZakresieKolejnosci):
    kolejnosc_gte = 0
    kolejnosc_lt = 5
    label = "Nazwisko i imię (od 1 do 5)"
    public = False
    field_name = "naz_im_1_5"


class TypOgolnyAutorQueryObject(NazwiskoIImieQueryObject):
    ops = [EQUAL, DIFFERENT, UNION]

    label = "Autor"
    typ_ogolny = const.TO_AUTOR
    field_name = "typ_og_autor"

    def real_query(self, value, operation):

        if operation in EQUALITY_OPS_ALL:
            ret = Q(
                autorzy__autor=value,
                autorzy__typ_odpowiedzialnosci__typ_ogolny=self.typ_ogolny,
            )

        elif operation in UNION_OPS_ALL:
            q = Autorzy.objects.filter(
                autor=value, typ_odpowiedzialnosci__typ_ogolny=self.typ_ogolny
            ).values("rekord_id")
            ret = Q(pk__in=q)

        else:
            raise UnknownOperation(operation)

        if operation in DIFFERENT_ALL:
            return ~ret

        return ret


class TypOgolnyRedaktorQueryObject(TypOgolnyAutorQueryObject):
    typ_ogolny = const.TO_REDAKTOR
    label = "Redaktor"
    field_name = "typ_og_redaktor"


class TypOgolnyTlumaczQueryObject(TypOgolnyAutorQueryObject):
    typ_ogolny = const.TO_TLUMACZ
    label = "Tłumacz"
    field_name = "typ_og_tlumacz"


class TypOgolnyRecenzentQueryObject(TypOgolnyAutorQueryObject):
    typ_ogolny = const.TO_RECENZENT
    label = "Recenzent"
    field_name = "typ_og_recenzent"


class DyscyplinaQueryObject(
    BppMultiseekVisibilityMixin, ForeignKeyDescribeMixin, AutocompleteQueryObject
):
    label = "Dyscyplina naukowa autora"
    type = AUTOCOMPLETE
    ops = [
        EQUAL_NONE,
    ]
    model = Dyscyplina_Naukowa
    field_name = "nazwa"
    url = "bpp:dyscyplina-autocomplete"

    def real_query(self, value, operation):
        if operation in EQUALITY_OPS_ALL:
            ret = Q(autorzy__dyscyplina_naukowa=value)
        else:
            raise UnknownOperation(operation)

        if operation in DIFFERENT_ALL:
            return ~ret
        return ret


class NazwaKonferencji(
    BppMultiseekVisibilityMixin, ForeignKeyDescribeMixin, AutocompleteQueryObject
):
    label = "Konferencja"
    type = AUTOCOMPLETE
    ops = EQUALITY_OPS_FEMALE
    model = Konferencja
    search_fields = ["nazwa"]
    field_name = "konferencja"
    url = "bpp:public-konferencja-autocomplete"


class ZewnetrznaBazaDanychQueryObject(
    BppMultiseekVisibilityMixin, ForeignKeyDescribeMixin, AutocompleteQueryObject
):
    label = "Zewnętrzna baza danych"
    field_name = "zewn_baza_danych"
    type = AUTOCOMPLETE
    ops = [
        EQUAL_FEMALE,
    ]
    model = Zewnetrzna_Baza_Danych
    search_fields = ["nazwa"]
    url = "bpp:zewnetrzna-baza-danych-autocomplete"

    def real_query(self, value, operation, validate_operation=True):
        if operation in EQUALITY_OPS_ALL:
            q = ZewnetrzneBazyDanychView.objects.filter(baza=value).values("rekord_id")
            ret = Q(pk__in=q)
        else:
            raise UnknownOperation(operation)
        return ret


EQUAL_PLUS_SUB_FEMALE = "równa+podrzędne"
EQUAL_PLUS_SUB_UNION_FEMALE = "równa+podrzędne+wspólna"


class JednostkaQueryObject(
    BppMultiseekVisibilityMixin, ForeignKeyDescribeMixin, AutocompleteQueryObject
):
    label = "Jednostka"
    type = AUTOCOMPLETE
    ops = [
        EQUAL_FEMALE,
        DIFFERENT_FEMALE,
        EQUAL_PLUS_SUB_FEMALE,
        UNION_FEMALE,
        EQUAL_PLUS_SUB_UNION_FEMALE,
    ]
    model = Jednostka
    search_fields = ["nazwa"]
    field_name = "jednostka"
    url = "bpp:jednostka-widoczna-autocomplete"

    def real_query(self, value, operation):
        if operation in EQUALITY_OPS_ALL:
            ret = Q(autorzy__jednostka=value)

        elif operation == EQUAL_PLUS_SUB_FEMALE:
            ret = Q(autorzy__jednostka__in=value.get_family())

        elif operation in EQUAL_PLUS_SUB_UNION_FEMALE:
            q = Autorzy.objects.filter(jednostka__in=value.get_family()).values(
                "rekord_id"
            )
            ret = Q(pk__in=q)

        elif operation in UNION_OPS_ALL:
            q = Autorzy.objects.filter(jednostka=value).values("rekord_id")
            ret = Q(pk__in=q)

        else:
            raise UnknownOperation(operation)

        if operation in DIFFERENT_ALL:
            return ~ret
        return ret


class AktualnaJednostkaAutoraQueryObject(JednostkaQueryObject):
    label = "Aktualna jednostka dowolnego autora"
    type = AUTOCOMPLETE
    ops = [
        EQUAL_FEMALE,
        DIFFERENT_FEMALE,
        EQUAL_PLUS_SUB_FEMALE,
    ]
    model = Jednostka
    search_fields = ["nazwa"]
    field_name = "aktualna_jednostka"
    url = "bpp:jednostka-widoczna-autocomplete"

    def real_query(self, value, operation):
        if operation in EQUALITY_OPS_ALL:
            ret = Q(autorzy__autor__aktualna_jednostka=value)

        elif operation == EQUAL_PLUS_SUB_FEMALE:
            ret = Q(autorzy__autor__aktualna_jednostka__in=value.get_family())

        elif operation in EQUAL_PLUS_SUB_UNION_FEMALE:
            q = Autorzy.objects.filter(
                autor__aktualna_jednostka__in=value.get_family()
            ).values("rekord_id")
            ret = Q(pk__in=q)

        elif operation in UNION_OPS_ALL:
            q = Autorzy.objects.filter(autor__aktualna_jednostka=value).values(
                "rekord_id"
            )
            ret = Q(pk__in=q)

        else:
            raise UnknownOperation(operation)

        if operation in DIFFERENT_ALL:
            return ~ret
        return ret


class PierwszaJednostkaQueryObject(JednostkaQueryObject):
    ops = [
        EQUAL_FEMALE,
        DIFFERENT_FEMALE,
        EQUAL_PLUS_SUB_FEMALE,
        UNION_FEMALE,
        EQUAL_PLUS_SUB_UNION_FEMALE,
    ]
    label = "Pierwsza jednostka"
    field_name = "pierwsza_jednostka"

    def real_query(self, value, operation):
        if operation in EQUALITY_OPS_ALL:
            ret = Q(autorzy__jednostka=value, autorzy__kolejnosc=0)

        elif operation == EQUAL_PLUS_SUB_FEMALE:
            ret = Q(autorzy__jednostka__in=value.get_family(), autorzy__kolejnosc=0)

        elif operation in EQUAL_PLUS_SUB_UNION_FEMALE:
            q = Autorzy.objects.filter(
                jednostka__in=value.get_family(), kolejnosc=0
            ).values("rekord_id")
            ret = Q(pk__in=q)

        elif operation in UNION_OPS_ALL:
            q = Autorzy.objects.filter(jednostka=value, kolejnosc=0).values("rekord_id")
            ret = Q(pk__in=q)

        else:
            raise UnknownOperation(operation)

        if operation in DIFFERENT_ALL:
            return ~ret
        return ret


class WydzialQueryObject(
    BppMultiseekVisibilityMixin, ForeignKeyDescribeMixin, AutocompleteQueryObject
):
    label = "Wydział"
    type = AUTOCOMPLETE
    ops = [EQUAL, DIFFERENT, UNION]
    model = Wydzial
    search_fields = ["nazwa"]
    field_name = "wydzial"
    url = "bpp:public-wydzial-autocomplete"

    def real_query(self, value, operation):
        if operation in EQUALITY_OPS_ALL:
            ret = Q(autorzy__jednostka__wydzial=value)

        elif operation in UNION_OPS_ALL:
            q = Autorzy.objects.filter(jednostka__wydzial=value).values("rekord_id")
            ret = Q(pk__in=q)

        else:
            raise UnknownOperation(operation)

        if operation in DIFFERENT_ALL:
            return ~ret

        return ret


class PierwszyWydzialQueryObject(WydzialQueryObject):
    label = "Pierwszy wydział"
    field_name = "pierwszy_wydzial"

    def real_query(self, value, operation):
        if operation in EQUALITY_OPS_ALL:
            ret = Q(autorzy__jednostka__wydzial=value, autorzy__kolejnosc=0)

        elif operation in UNION_OPS_ALL:
            q = Autorzy.objects.filter(jednostka__wydzial=value, kolejnosc=0).values(
                "rekord_id"
            )
            ret = Q(pk__in=q)

        else:
            raise UnknownOperation(operation)

        if operation in DIFFERENT_ALL:
            return ~ret

        return ret


class Typ_OdpowiedzialnosciQueryObject(BppMultiseekVisibilityMixin, QueryObject):
    label = "Typ odpowiedzialności"
    type = VALUE_LIST
    values = Typ_Odpowiedzialnosci.objects.all()
    ops = [EQUAL, DIFFERENT, UNION]
    field_name = "typ_odpowiedzialnosci"
    public = False

    def value_from_web(self, value):
        return Typ_Odpowiedzialnosci.objects.get(nazwa=value)

    def real_query(self, value, operation):
        if operation in EQUALITY_OPS_ALL:
            ret = Q(autorzy__typ_odpowiedzialnosci=value)

        elif operation in UNION_OPS_ALL:
            q = Autorzy.objects.filter(typ_odpowiedzialnosci=value).values("rekord_id")
            ret = Q(pk__in=q)

        else:
            raise UnknownOperation(operation)

        if operation in DIFFERENT_ALL:
            return ~ret

        return ret


class ZakresLatQueryObject(BppMultiseekVisibilityMixin, RangeQueryObject):
    label = "Zakres lat"
    field_name = "rok"
    bpp_multiseek_visibility_field_name = "zakres_lat"


class JezykQueryObject(BppMultiseekVisibilityMixin, QueryObject):
    label = "Język"
    type = VALUE_LIST
    ops = EQUALITY_OPS_MALE
    values = Jezyk.objects.all()
    field_name = "jezyk"

    def value_from_web(self, value):
        return Jezyk.objects.get(nazwa=value)


class RokQueryObject(BppMultiseekVisibilityMixin, IntegerQueryObject):
    label = "Rok"
    field_name = "rok"


class ImpactQueryObject(BppMultiseekVisibilityMixin, DecimalQueryObject):
    label = "Impact factor"
    field_name = "impact_factor"


class LiczbaCytowanQueryObject(BppMultiseekVisibilityMixin, IntegerQueryObject):
    label = "Liczba cytowań"
    field_name = "liczba_cytowan"


class LiczbaAutorowQueryObject(BppMultiseekVisibilityMixin, IntegerQueryObject):
    label = "Liczba autorów"
    field_name = "liczba_autorow"


class KCImpactQueryObject(ImpactQueryObject):
    field_name = "kc_impact_factor"
    label = "KC: Impact factor"
    public = False


class PunktacjaWewnetrznaEnabledMixin:
    def option_enabled(self, request):
        return settings.UZYWAJ_PUNKTACJI_WEWNETRZNEJ


class PunktacjaWewnetrznaQueryObject(
    BppMultiseekVisibilityMixin, PunktacjaWewnetrznaEnabledMixin, DecimalQueryObject
):
    label = "Punktacja wewnętrzna"
    field_name = "punktacja_wewnetrzna"


class PunktacjaSNIP(BppMultiseekVisibilityMixin, DecimalQueryObject):
    label = "Punktacja SNIP"
    field_name = "punktacja_snip"


class PunktyKBNQueryObject(BppMultiseekVisibilityMixin, DecimalQueryObject):
    label = "Punkty PK"
    field_name = "punkty_kbn"


class KCPunktyKBNQueryObject(PunktyKBNQueryObject):
    label = "KC: Punkty PK"
    field_name = "kc_punkty_kbn"
    public = False


class IndexCopernicusQueryObject(BppMultiseekVisibilityMixin, DecimalQueryObject):
    label = "Index Copernicus"
    field_name = "index_copernicus"

    def option_enabled(self):
        u = Uczelnia.objects.get_default()
        if u is not None:
            return u.pokazuj_index_copernicus
        return True


class LiczbaZnakowWydawniczychQueryObject(
    BppMultiseekVisibilityMixin, IntegerQueryObject
):
    label = "Liczba znaków wydawniczych"
    field_name = "liczba_znakow_wydawniczych"


class TypRekorduObject(BppMultiseekVisibilityMixin, ValueListQueryObject):
    label = "Typ rekordu"
    field_name = "typ_rekordu"
    values = ["publikacje", "streszczenia", "inne"]
    ops = [EQUAL, DIFFERENT]

    def value_from_web(self, value):
        if value not in self.values:
            return
        return value

    def real_query(self, value, operation):
        if value == "publikacje":
            charaktery = Charakter_Formalny.objects.filter(publikacja=True)
        elif value == "streszczenia":
            charaktery = Charakter_Formalny.objects.filter(streszczenie=True)
        else:
            charaktery = (
                Charakter_Formalny.objects.all()
                .exclude(streszczenie=True)
                .exclude(publikacja=True)
            )

        q = Q(**{"charakter_formalny__in": charaktery})
        if operation == DIFFERENT:
            return ~q
        return q


class CharakterOgolnyQueryObject(BppMultiseekVisibilityMixin, ValueListQueryObject):
    label = "Charakter formalny ogólny"
    field_name = "charakter_formalny_ogolny"
    values = ["artykuł", "rozdział", "książka", "inne"]
    ops = [EQUAL, DIFFERENT]

    def value_from_web(self, value):
        if value not in self.values:
            return
        return value

    def real_query(self, value, operation):
        if value == "artykuł":
            charaktery = Charakter_Formalny.objects.filter(
                charakter_ogolny=const.CHARAKTER_OGOLNY_ARTYKUL
            )
        elif value == "rozdział":
            charaktery = Charakter_Formalny.objects.filter(
                charakter_ogolny=const.CHARAKTER_OGOLNY_ROZDZIAL
            )
        elif value == "książka":
            charaktery = Charakter_Formalny.objects.filter(
                charakter_ogolny=const.CHARAKTER_OGOLNY_KSIAZKA
            )
        elif value == "inne":
            charaktery = Charakter_Formalny.objects.filter(
                charakter_ogolny=const.CHARAKTER_OGOLNY_INNE
            )
        else:
            raise NotImplementedError()

        q = Q(**{"charakter_formalny__in": charaktery})
        if operation == DIFFERENT:
            return ~q
        return q


class CharakterFormalnyQueryObject(
    BppMultiseekVisibilityMixin, TreeNodeChoiceFieldMixin, ValueListQueryObject
):
    field_name = "charakter_formalny"
    label = "Charakter formalny"

    start_level = 0

    def _values(self):
        for elem in self.queryset:
            yield self.label_from_instance(elem)

    values = property(_values)

    def value_from_web(self, value):
        return Charakter_Formalny.objects.get(nazwa=value.lstrip("-").lstrip(" "))

    def __init__(self, *args, **kwargs):
        ValueListQueryObject.__init__(self, *args, **kwargs)

        self.level_indicator = kwargs.pop("level_indicator", DEFAULT_LEVEL_INDICATOR)

    @property
    def queryset(self):
        queryset = Charakter_Formalny.objects.all()
        # if a queryset is supplied, enforce ordering
        if hasattr(queryset, "model"):
            mptt_opts = queryset.model._mptt_meta
            queryset = queryset.order_by(mptt_opts.tree_id_attr, mptt_opts.left_attr)
        return queryset
        # self.queryset = queryset

    def real_query(self, value, operation, validate_operation=True):
        ret = None

        if operation in [str(x) for x in EQUALITY_OPS_ALL]:
            ret = Q(
                **{self.field_name + "__in": value.get_descendants(include_self=True)}
            )

        else:
            if validate_operation:
                raise UnknownOperation(operation)

        if operation in DIFFERENT_ALL:
            return ~ret

        return ret


class OpenaccessWersjaTekstuQueryObject(
    BppMultiseekVisibilityMixin, ValueListQueryObject
):
    field_name = "openaccess_wersja_tekstu"
    values = Wersja_Tekstu_OpenAccess.objects.all()
    label = "OpenAccess: wersja tekstu"

    def value_from_web(self, value):
        return Wersja_Tekstu_OpenAccess.objects.get(nazwa=value)


class OpenaccessLicencjaQueryObject(BppMultiseekVisibilityMixin, ValueListQueryObject):
    field_name = "openaccess_licencja"
    values = Licencja_OpenAccess.objects.all()
    label = "OpenAccess: licencja"

    def value_from_web(self, value):
        return Licencja_OpenAccess.objects.get(nazwa=value)


class OpenaccessCzasPublikacjiQueryObject(
    BppMultiseekVisibilityMixin, ValueListQueryObject
):
    field_name = "openaccess_czas_publikacji"
    values = Czas_Udostepnienia_OpenAccess.objects.all()
    label = "OpenAccess: czas udostępnienia"

    def value_from_web(self, value):
        return Czas_Udostepnienia_OpenAccess.objects.get(nazwa=value)


class TypKBNQueryObject(BppMultiseekVisibilityMixin, ValueListQueryObject):
    field_name = "typ_kbn"
    values = Typ_KBN.objects.all()
    label = "Typ KBN"

    def value_from_web(self, value):
        return Typ_KBN.objects.get(nazwa=value)


class ZrodloQueryObject(BppMultiseekVisibilityMixin, AutocompleteQueryObject):
    label = "Źródło"
    ops = EQUALITY_OPS_NONE
    model = Zrodlo
    field_name = "zrodlo"
    url = "bpp:zrodlo-autocomplete"


class RecenzowanaQueryObject(BppMultiseekVisibilityMixin, BooleanQueryObject):
    ops = EQUALITY_OPS_NONE
    field_name = "recenzowana"
    label = "Praca recenzowana"


class BazaWOS(BppMultiseekVisibilityMixin, BooleanQueryObject):
    ops = EQUALITY_OPS_NONE
    field_name = "konferencja__baza_wos"
    label = "Konferencja w bazie Web of Science"


class BazaSCOPUS(BppMultiseekVisibilityMixin, BooleanQueryObject):
    ops = EQUALITY_OPS_NONE
    field_name = "konferencja__baza_scopus"
    label = "Konferencja w bazie Scopus"


class RodzajKonferenckjiQueryObject(BppMultiseekVisibilityMixin, ValueListQueryObject):
    label = "Rodzaj konferencji"
    field_name = "rodzaj_konferencji"
    values = ["krajowa", "międzynarodowa", "lokalna"]

    def value_from_web(self, value):
        if value not in self.values:
            return
        return value

    def real_query(self, value, operation):
        if value == "krajowa":
            tk = Konferencja.TK_KRAJOWA
        elif value == "międzynarodowa":
            tk = Konferencja.TK_MIEDZYNARODOWA
        else:
            tk = Konferencja.TK_LOKALNA

        q = Q(**{"konferencja__typ_konferencji": tk})
        if operation == DIFFERENT:
            return ~q
        return q


class ObcaJednostkaQueryObject(BppMultiseekVisibilityMixin, BooleanQueryObject):
    label = "Obca jednostka"
    field_name = "obca_jednostka"
    ops = EQUALITY_OPS_FEMALE
    public = False

    def real_query(self, value, operation):
        value = not value

        if operation in EQUALITY_OPS_ALL:
            ret = Q(autorzy__jednostka__skupia_pracownikow=value)
        else:
            raise UnknownOperation(operation)

        if operation in DIFFERENT_ALL:
            return ~ret

        return ret


class AfiliujeQueryObject(BppMultiseekVisibilityMixin, BooleanQueryObject):
    label = "Afiliuje"
    field_name = "afiliuje"
    ops = [
        EQUAL,
    ]
    public = False

    def real_query(self, value, operation):
        if operation in EQUALITY_OPS_ALL:
            ret = Q(autorzy__afiliuje=value)
        else:
            raise UnknownOperation(operation)

        if operation in DIFFERENT_ALL:
            return ~ret

        return ret


class DyscyplinaUstawionaQueryObject(BppMultiseekVisibilityMixin, BooleanQueryObject):
    label = "Dyscyplina ustawiona"
    field_name = "dyscyplina_ustawiona"
    ops = EQUALITY_OPS_FEMALE
    public = False

    def real_query(self, value, operation):
        if operation in EQUALITY_OPS_ALL:
            if value:
                ret = ~Q(autorzy__dyscyplina_naukowa=None)
            else:
                ret = Q(autorzy__dyscyplina_naukowa=None)
        else:
            raise UnknownOperation(operation)

        if operation in DIFFERENT_ALL:
            return ~ret

        return ret


class StronaWWWUstawionaQueryObject(BppMultiseekVisibilityMixin, BooleanQueryObject):
    label = "Strona WWW ustawiona"
    field_name = "www_ustawiona"
    ops = [
        EQUAL_FEMALE,
    ]
    public = False

    def real_query(self, value, operation):
        if value:
            ret = Q(
                Q(~Q(public_www="") & Q(public_www__isnull=False))
                | Q(~Q(www="") & Q(www__isnull=False))
            )
        else:
            ret = Q(
                Q(Q(public_www="") | Q(public_www__isnull=True))
                & Q(Q(www="") | Q(www__isnull=True))
            )

        return ret


class LicencjaOpenAccessUstawionaQueryObject(
    BppMultiseekVisibilityMixin, BooleanQueryObject
):
    label = "OpenAccess: licencja ustawiona"
    field_name = "lo_ustawiona"
    ops = EQUALITY_OPS_FEMALE
    public = False

    def real_query(self, value, operation):
        if operation in EQUALITY_OPS_ALL:
            if value:
                ret = Q(openaccess_licencja__isnull=False)
            else:
                ret = Q(openaccess_licencja__isnull=True)
        else:
            raise UnknownOperation(operation)

        if operation in DIFFERENT_ALL:
            return ~ret

        return ret


class PublicDostepDniaQueryObject(BppMultiseekVisibilityMixin, BooleanQueryObject):
    label = "Dostęp dnia ustawiony"
    field_name = "public_dostep_dnia"
    public = False

    def real_query(self, value, operation):
        if operation in EQUALITY_OPS_ALL:
            if value:
                ret = ~Q(public_dostep_dnia=None)
            else:
                ret = Q(public_dostep_dnia=None)
        else:
            raise UnknownOperation(operation)

        if operation in DIFFERENT_ALL:
            return ~ret

        return ret


multiseek_fields = [
    TytulPracyQueryObject(),
    NazwiskoIImieQueryObject(),
    JednostkaQueryObject(),
    WydzialQueryObject(),
    Typ_OdpowiedzialnosciQueryObject(),
    TypOgolnyAutorQueryObject(),
    TypOgolnyRedaktorQueryObject(),
    TypOgolnyTlumaczQueryObject(),
    TypOgolnyRecenzentQueryObject(),
    ZakresLatQueryObject(),
    JezykQueryObject(),
    RokQueryObject(),
    TypRekorduObject(),
    CharakterFormalnyQueryObject(),
    CharakterOgolnyQueryObject(),
    TypKBNQueryObject(),
    ZrodloQueryObject(),
    WydawnictwoNadrzedneQueryObject(),
    PierwszeNazwiskoIImie(),
    PierwszaJednostkaQueryObject(),
    PierwszyWydzialQueryObject(),
    NazwiskoIImie1do3(),
    NazwiskoIImie1do5(),
    OstatnieNazwiskoIImie(),
    ORCIDQueryObject(),
    ImpactQueryObject(),
    LiczbaCytowanQueryObject(),
    LiczbaAutorowQueryObject(),
    PunktyKBNQueryObject(),
    IndexCopernicusQueryObject(),
    PunktacjaSNIP(),
    PunktacjaWewnetrznaQueryObject(),
    KCImpactQueryObject(),
    KCPunktyKBNQueryObject(),
    InformacjeQueryObject(),
    SzczegolyQueryObject(),
    UwagiQueryObject(),
    SlowaKluczoweQueryObject(),
    AdnotacjeQueryObject(),
    DataUtworzeniaQueryObject(),
    OstatnioZmieniony(),
    RecenzowanaQueryObject(),
    LiczbaZnakowWydawniczychQueryObject(),
    NazwaKonferencji(),
    RodzajKonferenckjiQueryObject(),
    BazaWOS(),
    BazaSCOPUS(),
    OpenaccessWersjaTekstuQueryObject(),
    OpenaccessLicencjaQueryObject(),
    OpenaccessCzasPublikacjiQueryObject(),
    DyscyplinaQueryObject(),
    ZewnetrznaBazaDanychQueryObject(),
    ObcaJednostkaQueryObject(),
    AfiliujeQueryObject(),
    DyscyplinaUstawionaQueryObject(),
    LicencjaOpenAccessUstawionaQueryObject(),
    PublicDostepDniaQueryObject(),
    StronaWWWUstawionaQueryObject(),
    DOIQueryObject(),
    AktualnaJednostkaAutoraQueryObject(),
]


class PunktacjaWewnetrznaReportType(PunktacjaWewnetrznaEnabledMixin, ReportType):
    pass



import os

from django.conf import settings
from django.http.response import FileResponse, HttpResponse, HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.template.context import RequestContext
from django.views.generic import FormView, TemplateView
from django.views.generic.detail import DetailView
from django_tables2.export.export import TableExport
from flexible_reports.adapters.django_tables2 import as_docx, as_tablib_databook
from flexible_reports.models.report import Report

from formdefaults.helpers import FormDefaultsMixin
from .forms import (
    AutorRaportForm,
    JednostkaRaportForm,
    UczelniaRaportForm,
    WydzialRaportForm,
)

from bpp.const import GR_RAPORTY_WYSWIETLANIE
from bpp.models import Uczelnia
from bpp.models.autor import Autor
from bpp.models.cache import Rekord
from bpp.models.struktura import Jednostka, Wydzial
from bpp.views.mixins import UczelniaSettingRequiredMixin


class BaseFormView(FormDefaultsMixin, FormView):
    template_name = "nowe_raporty/formularz.html"
    title = "Raporty"

    def form_valid(self, form):
        d = form.cleaned_data
        return HttpResponseRedirect(
            f"./{ d['obiekt'].pk }/{ d['od_roku'] }/{ d['do_roku'] }/?"
            f"_export={ d['_export'] }&"
            f"_tzju={d['tylko_z_jednostek_uczelni']}"
        )

    def get_context_data(self, **kwargs):
        kwargs["title"] = self.title
        kwargs["report"] = get_object_or_404(Report, slug=self.report_slug)
        return super().get_context_data(**kwargs)


class BaseRaportAuthMixin(UczelniaSettingRequiredMixin):
    """Wspólny mixin - bazowa klasa autoryzująca dostęp do raportów.

    Wymaga ustawienia obiektu uczelnia ``pokazuj_raport_x`` oraz wymaga
    odpowiedniej grupy czyli ``generowanie_raportow``.

    """

    group_required = GR_RAPORTY_WYSWIETLANIE


class AutorRaportAuthMixin(BaseRaportAuthMixin):
    uczelnia_attr = "pokazuj_raport_autorow"


class JednostkaRaportAuthMixin(BaseRaportAuthMixin):
    uczelnia_attr = "pokazuj_raport_jednostek"


class WydzialRaportAuthMixin(BaseRaportAuthMixin):
    uczelnia_attr = "pokazuj_raport_wydzialow"


class UczelniaRaportAuthMixin(BaseRaportAuthMixin):
    uczelnia_attr = "pokazuj_raport_uczelni"


class AutorRaportFormView(AutorRaportAuthMixin, BaseFormView):
    form_class = AutorRaportForm
    title = "Raport autorów"
    report_slug = "raport-autorow"

    def form_valid(self, form):
        d = form.cleaned_data
        return HttpResponseRedirect(
            f"./{ d['obiekt'].pk }/{ d['od_roku'] }/{ d['do_roku'] }/?"
            f"_export={ d['_export'] }&"
            f"_tzju={ d['tylko_z_jednostek_uczelni'] }"
        )


class JednostkaRaportFormView(JednostkaRaportAuthMixin, BaseFormView):
    report_slug = "raport-jednostek"
    form_class = JednostkaRaportForm
    title = "Raport jednostek"


class UczelniaRaportFormView(UczelniaRaportAuthMixin, BaseFormView):
    report_slug = "raport-uczelni"
    form_class = UczelniaRaportForm
    title = "Raport uczelni"

    def form_valid(self, form):
        d = form.cleaned_data
        return HttpResponseRedirect(
            f"./{ d['od_roku'] }/{ d['do_roku'] }/?"
            f"_export={ d['_export'] }&"
            f"_tzju={d['tylko_z_jednostek_uczelni']}"
        )


class WydzialRaportFormView(WydzialRaportAuthMixin, BaseFormView):
    report_slug = "raport-wydzialow"
    form_class = WydzialRaportForm
    title = "Raport wydziałów"


class BaseGenerujView(TemplateView):
    def get_context_data(self, **kwargs):
        return kwargs


class GenerujRaportBase(DetailView):
    template_name = "nowe_raporty/generuj.html"

    @property
    def okres(self):
        if self.kwargs["od_roku"] == self.kwargs["do_roku"]:
            return self.kwargs["od_roku"]
        else:
            return "{}-{}".format(self.kwargs["od_roku"], self.kwargs["do_roku"])

    @property
    def title(self):
        return f"Raport dla { self.object } za { self.okres }"

    def get_context_data(self, **kwargs):
        from flexible_reports.models import Report

        try:
            report = Report.objects.get(slug=self.report_slug)
        except Report.DoesNotExist:
            report = None

        if report:
            base_queryset = self.get_base_queryset()

            base_queryset = base_queryset.filter(
                rok__gte=self.kwargs["od_roku"], rok__lte=self.kwargs["do_roku"]
            )

            uczelnia = Uczelnia.objects.get_for_request(self.request)
            if uczelnia is not None:
                ukryte_statusy = uczelnia.ukryte_statusy("raporty")
                if ukryte_statusy:
                    base_queryset = base_queryset.exclude(
                        status_korekty_id__in=ukryte_statusy
                    )

            base_queryset = base_queryset.select_related(
                "typ_kbn", "charakter_formalny"
            )

            report.set_base_queryset(base_queryset)

            report.set_context(
                {
                    "obiekt": self.object,
                    "punktuj_monografie": settings.PUNKTUJ_MONOGRAFIE,
                }
            )

        kwargs["report"] = report
        kwargs["od_roku"] = self.kwargs["od_roku"]
        kwargs["do_roku"] = self.kwargs["do_roku"]
        kwargs["title"] = self.title
        kwargs["form_link"] = self.form_link
        kwargs["form_title"] = self.form_title
        return super().get_context_data(**kwargs)

    def as_docx_response(self, report, parent_context, filename=None):
        data = as_docx(report, parent_context)

        response = FileResponse(
            data,
            content_type="application/vnd.openxmlformats-officedocument.wordprocessingml.document",
        )

        if filename is None:
            filename = report.title + ".docx"

        response["Content-Disposition"] = f'attachment; filename="{filename}"'

        response["Content-Length"] = os.stat(data.name).st_size
        return response

    def as_xlsx_response(self, report, parent_context, filename=None):
        response = HttpResponse(content_type=TableExport.FORMATS[TableExport.XLSX])
        if filename is None:
            filename = report.title + ".xlsx"

        response["Content-Disposition"] = f'attachment; filename="{filename}"'

        xlsx = as_tablib_databook(report, parent_context)

        # Patch dla błędu "Invlaid character / found in sheet title"
        for sheet in xlsx.sheets():
            sheet.title = sheet.title.replace("/", "-")

        data = xlsx.export(TableExport.XLSX)
        response["Content-Length"] = len(data)
        response.write(data)
        return response

    def render_to_response(self, context, **response_kwargs):
        _export = self.request.GET.get("_export")

        if _export in ("docx", "xlsx"):
            context["request"] = self.request
            parent_context = RequestContext(self.request, context)
            fun = getattr(self, "as_%s_response" % _export)
            return fun(
                context["report"], parent_context, filename=self.title + "." + _export
            )

        return super().render_to_response(context, **response_kwargs)


class GenerujRaportDlaAutora(AutorRaportAuthMixin, GenerujRaportBase):
    report_slug = "raport-autorow"
    form_link = "nowe_raporty:autor_form"
    form_title = "Raport autorów"
    model = Autor

    def get_base_queryset(self):
        if self.request.GET.get("_tzju", "True") == "True":
            return Rekord.objects.prace_autora_z_afiliowanych_jednostek(self.object)

        return Rekord.objects.prace_autora(self.object)


class GenerujRaportDlaJednostki(JednostkaRaportAuthMixin, GenerujRaportBase):
    report_slug = "raport-jednostek"
    form_link = "nowe_raporty:jednostka_form"
    form_title = "Raport jednostek"
    model = Jednostka

    def get_base_queryset(self):
        if self.request.GET.get("_tzju", "True") == "True":
            return Rekord.objects.prace_jednostki(self.object, afiliowane=True)
        return Rekord.objects.prace_jednostki(self.object)


class GenerujRaportDlaWydzialu(WydzialRaportAuthMixin, GenerujRaportBase):
    report_slug = "raport-wydzialow"
    form_link = "nowe_raporty:wydzial_form"
    form_title = "Raport wydziałów"
    model = Wydzial

    def get_base_queryset(self):
        if self.request.GET.get("_tzju", "True") == "True":
            return Rekord.objects.prace_wydzialu(self.object, afiliowane=True)
        return Rekord.objects.prace_wydzialu(self.object)


class GenerujRaportDlaUczelni(UczelniaRaportAuthMixin, GenerujRaportBase):
    report_slug = "raport-uczelni"
    form_link = "nowe_raporty:uczelnia_form"
    form_title = "Raport uczelni"
    model = Uczelnia

    def get_object(self, queryset=None):
        return Uczelnia.objects.get_for_request(self.request)

    def get_base_queryset(self):
        if self.request.GET.get("_tzju", "True") == "True":
            return Rekord.objects.filter(autorzy__afiliuje=True)
        return Rekord.objects.all()



from django import forms
from django.db import models
from django.db.models import JSONField

from import_common.core import matchuj_zrodlo
from import_common.models import ImportOperation, ImportRowMixin

from bpp.models import (
    IF_DECIMAL_PLACES,
    IF_MAX_DIGITS,
    ImpactFactorField,
    Punktacja_Zrodla,
    Zrodlo,
)


class ImportListRowValidationForm(forms.Form):
    full_journal_title = forms.CharField(max_length=512)
    journal_impact_factor = forms.DecimalField(
        max_digits=IF_MAX_DIGITS, decimal_places=IF_DECIMAL_PLACES
    )


class ImportListIf(ImportOperation):
    rok = models.PositiveSmallIntegerField()

    try_names = ["full_journal_title", "journal_impact_factor"]
    banned_names = []
    min_points = 2

    validation_form_class = ImportListRowValidationForm

    def import_single_row(self, xls_data, cleaned_data):
        zrodlo = matchuj_zrodlo(cleaned_data.get("full_journal_title"))
        impact_factor = cleaned_data.get("journal_impact_factor")

        res = ImportListIfRow(
            parent=self,
            dane_z_xls=xls_data,
            zrodlo=zrodlo,
            impact_factor=impact_factor,
        )

        if zrodlo is None:
            res.save()
            return

        try:
            pz = zrodlo.punktacja_zrodla_set.get(rok=self.rok)
        except Punktacja_Zrodla.DoesNotExist:
            pz = zrodlo.punktacja_zrodla_set.create(rok=self.rok)

        if pz.impact_factor != impact_factor:
            pz.impact_factor = impact_factor
            pz.save()

            res.zintegrowano = True

        res.save()

    def on_reset(self):
        self.get_details_set().delete()

    def get_details_set(self):
        return self.importlistifrow_set.all().select_related("zrodlo")


class ImportListIfRow(ImportRowMixin, models.Model):
    parent = models.ForeignKey(ImportListIf, on_delete=models.CASCADE)
    dane_z_xls = JSONField()

    zrodlo = models.ForeignKey(Zrodlo, null=True, blank=True, on_delete=models.SET_NULL)
    impact_factor = ImpactFactorField(null=True, blank=True)

    zintegrowano = models.BooleanField(default=False)



from rest_framework import serializers

from bpp.models import Czas_Udostepnienia_OpenAccess


class Czas_Udostepnienia_OpenAccess_Serializer(serializers.HyperlinkedModelSerializer,):
    class Meta:
        model = Czas_Udostepnienia_OpenAccess
        fields = ["id", "nazwa", "skrot"]



# -*- coding: utf-8 -*-


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0042_auto_20160802_2346'),
    ]

    operations = [
        migrations.AddField(
            model_name='jednostka',
            name='obca_jednostka',
            field=models.BooleanField(default=False, help_text=b'Zaznacz dla jednostki skupiaj\xc4\x85cej autor\xc3\xb3w nieindeksowanych,\n        np. dla "Obca jednostka" lub "Doktoranci".\n\n                <br/><br/>\n        Je\xc5\xbceli wi\xc4\x99cej, ni\xc5\xbc jedna jednostka w bazie danych ma to pole oznaczone jako \'PRAWDA\', autorzy\n        przypisywani b\xc4\x99d\xc4\x85 do jednostki obcej o ni\xc5\xbcszym numerze unikalnego identyfikatora (ID).\n        '),
        ),
        migrations.AddField(
            model_name='jednostka',
            name='wirtualna',
            field=models.BooleanField(default=False, help_text=b'Jednostka wirtualna to jednostka nie maj\xc4\x85ca odzwierciedlenia\n        w strukturach uczelni, utworzona jedynie na potrzeby bazy danych. Przyk\xc5\x82adowo, mo\xc5\xbce by\xc4\x87\n        to "Obca jednostka" - dla autor\xc3\xb3w nieindeksowanych, lub te\xc5\xbc mo\xc5\xbce by\xc4\x87 to jednostka "Doktoranci".\n        Tego typu jednostka musi by\xc4\x87 zarz\xc4\x85dzana r\xc4\x99cznie od strony strukturalnej - nie b\xc4\x99dzie\n        ukrywana, usuwana ani zmieniana przez procedury importuj\xc4\x85ce dane z zewn\xc4\x99trznych system\xc3\xb3w. '),
        ),
        migrations.AddField(
            model_name='wydzial',
            name='wirtualny',
            field=models.BooleanField(default=False, help_text=b'Wydzia\xc5\x82 wirtualny to wydzia\xc5\x82 nie maj\xc4\x85cy odzwierciedlenia\n        w strukturach uczelni, utworzony na potrzeby bazy danych. Przyk\xc5\x82adowo, mo\xc5\xbce by\xc4\x87\n        to wydzia\xc5\x82 skupiaj\xc4\x85cy jednostki dawne lub jednostki typu "Obca jednostka", "Doktoranci".\n        Tego typu wydzia\xc5\x82 musi by\xc4\x87 zarz\xc4\x85dzany r\xc4\x99cznie - nie b\xc4\x99dzie usuwany, ukrywany ani aktualizowany\n        przez procedury importuj\xc4\x85ce dane z zewn\xc4\x99trznych system\xc3\xb3w. '),
        ),
    ]



# Generated by Django 3.0.11 on 2021-02-28 16:39

import django.contrib.postgres.fields
import taggit.managers
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("taggit", "0003_taggeditem_add_unique_index"),
        ("bpp", "0239_egeria_import_new_autor_flds"),
    ]

    operations = [
        migrations.CreateModel(
            name="SlowaKluczoweView",
            fields=[
                (
                    "id",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.IntegerField(),
                        primary_key=True,
                        serialize=False,
                        size=3,
                    ),
                ),
            ],
            options={
                "db_table": "bpp_slowa_kluczowe_view",
                "managed": False,
            },
        ),
        migrations.AlterField(
            model_name="patent",
            name="slowa_kluczowe",
            field=taggit.managers.TaggableManager(
                blank=True,
                help_text="Lista słów kluczowych, oddzielonych przecinkiem.",
                through="taggit.TaggedItem",
                to="taggit.Tag",
                verbose_name="Słowa kluczowe",
            ),
        ),
        migrations.AlterField(
            model_name="praca_doktorska",
            name="slowa_kluczowe",
            field=taggit.managers.TaggableManager(
                blank=True,
                help_text="Lista słów kluczowych, oddzielonych przecinkiem.",
                through="taggit.TaggedItem",
                to="taggit.Tag",
                verbose_name="Słowa kluczowe",
            ),
        ),
        migrations.AlterField(
            model_name="praca_habilitacyjna",
            name="slowa_kluczowe",
            field=taggit.managers.TaggableManager(
                blank=True,
                help_text="Lista słów kluczowych, oddzielonych przecinkiem.",
                through="taggit.TaggedItem",
                to="taggit.Tag",
                verbose_name="Słowa kluczowe",
            ),
        ),
        migrations.AlterField(
            model_name="wydawnictwo_ciagle",
            name="slowa_kluczowe",
            field=taggit.managers.TaggableManager(
                blank=True,
                help_text="Lista słów kluczowych, oddzielonych przecinkiem.",
                through="taggit.TaggedItem",
                to="taggit.Tag",
                verbose_name="Słowa kluczowe",
            ),
        ),
        migrations.AlterField(
            model_name="wydawnictwo_zwarte",
            name="slowa_kluczowe",
            field=taggit.managers.TaggableManager(
                blank=True,
                help_text="Lista słów kluczowych, oddzielonych przecinkiem.",
                through="taggit.TaggedItem",
                to="taggit.Tag",
                verbose_name="Słowa kluczowe",
            ),
        ),
    ]



import numpy
import pygad

from .sumator_base import SumatorBase


class FitnessFuncMixin:
    def fitness_func(self, lista_prac):
        self.zeruj()
        for praca_idx in lista_prac:
            praca_tuple = self.lista_prac_by_index.get(praca_idx, None)
            assert praca_tuple is not None, "Brak pracy o indeksie"
            if self.czy_moze_przejsc(praca_tuple):
                self.zsumuj_pojedyncza_prace(praca_tuple, praca_idx)
        return self.suma_pkd


class GenetycznySumator(FitnessFuncMixin, SumatorBase):
    def __init__(self, lista_prac_by_index, *args, **kw):
        super().__init__(*args, **kw)
        self.lista_prac_by_index = lista_prac_by_index


def fitness_wrapper(lista_prac):
    return fitness_wrapper.instancja_genetycznego_sumatora.fitness_func(lista_prac)


def multiprocessing_gad_pool_initializer(
    lista_prac_by_index,
    liczba_2_2_n,
    liczba_0_8_n,
    maks_pkt_aut_calosc,
    maks_pkt_aut_monografie,
):
    """Ta funkcja wywoływana jest przy inicjalizacji multiprocessing.Pool.

    Tworzy ona instancję sumatora genetycznego a następnie zapisuje go jako atrybut
    funkcji fitness_wrapper (coby nie używać "global").

    W ten sposób funkcja fitness_wrapper
    może wywołać później funkcję sumatora genetycznego fitness_func celem obliczenia punktacji
    dla populacji w sposób wielowątkowy."""
    sumator = GenetycznySumator(
        lista_prac_by_index=lista_prac_by_index,
        liczba_2_2_n=liczba_2_2_n,
        liczba_0_8_n=liczba_0_8_n,
        maks_pkt_aut_calosc=maks_pkt_aut_calosc,
        maks_pkt_aut_monografie=maks_pkt_aut_monografie,
    )

    fitness_wrapper.instancja_genetycznego_sumatora = sumator


class MultiprocessingGAD(pygad.GA):
    def __init__(self, pool, *args, **kw):
        assert (
            "fitness_func" not in kw
        ), "Argument nie obsługiwany - fitness_func jest ustalone na sztywno w tej klasie"
        super().__init__(fitness_func=GenetycznySumator.fitness_func, *args, **kw)
        self.pool = pool

    def cal_pop_fitness(self):
        pop_fitness = self.pool.map(fitness_wrapper, self.population)
        pop_fitness = numpy.array(pop_fitness)
        return pop_fitness
